using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 

namespace SOAPClasses
{
   /// <summary>
   /// This class have examples how to work with alertm.
   /// </summary>
   /// <remarks>
   ///		<para>The current class includes:</para>
   ///		<list type="bullet">
   ///			<item><see cref="getAlerts" /> to get alerts.</item>
   ///			<item><see cref="subscribe" /> to subscribe on alerts.</item>
   ///          <item><see cref="unsubscribe" /> to unsubscribe from alerts.</item>
   ///     </list>
   /// </remarks>
   public class Alertm 
    {
        alertmBinding alertm;
        Binder binder;

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public Alertm(Binder _binder)
        {
            binder = _binder;
        }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
       public Alertm(string url, string session)
        {
            binder = new Binder(url, session);
        }
              
        ///<summary>
        /// Method to get all alerts
        /// <param name="ve_eid">VE from which you want to get alerts</param>
        /// <returns>
        /// <para>List of alerts.</para>
        /// </returns>
        /// <example>
        /// <para>Example: getAlerts(ve_list)</para>
        /// </example>
        ///</summary>
       public string getAlerts(string ve_eid)
       {
           try
           {
               if (alertm == null) alertm  = (alertmBinding)binder.InitBinding(typeof(alertmBinding));
               
               get_alerts alerts = new get_alerts();
              
              // eid_listType eidlist = new eid_listType();
               //Set VE list 
               string list_result = "";
               alerts.eid_list = new string[] { ve_eid };
               foreach (eventType al_event in alertm.get_alerts(alerts))
               {
                   list_result+="Data: \n";
                   resource_alertType res_data = (resource_alertType)al_event.data.event_data;
                   list_result+="  Class: "+res_data.@class+"\n"+
                   //Get counter
                   "  Counter: " + res_data.counter+"\n"+
                   //Get eid
                   "  Eid: " + res_data.eid +"\n"+
                   //Get instance
                   "  Instance: " + res_data.instance+"\n"+
                   //Get type
                   "  Type: " + res_data.type.ToString()+"\n"+
                   //Get current value
                   "  Cur: " + res_data.cur +"\n"+
                   //Get hard limit
                   "  Hard: " + res_data.hard+"\n"+
                   //Get soft limit
                   "  Soft: " + res_data.soft+"\n"+
                   //Get event name
                   "Name: " + al_event.info.name + "\n" +
                   //Get count
                   "Count: " + al_event.count.ToString()+"\n"+
                   //Get event category
                   "Category: " + al_event.category+"\n"+
                   //Get event message
                   "Message: " + System.Text.ASCIIEncoding.ASCII.GetString(al_event.info.message)+"\n"+
                   //Get parameters
                   "Parameters:";
                   getParams(al_event.info.parameter,ref list_result);
               }
               return list_result;
               
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }
       string ss = " ";
       void getParams(infoType[] parameter,ref string list)
       {
         foreach (infoType param in parameter) 
         {
             list+=ss+"Message: " + System.Text.ASCIIEncoding.ASCII.GetString(param.message)+
             "   Info name: " + param.name+"\n";
            
              if (param.parameter != null)
                getParams(param.parameter,ref list);
         }
         
       }

      
    }
}
