using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 

namespace SOAPClasses
{
   /// <summary>
   /// This class have examples how to work with filer.
   /// </summary>
   /// <remarks>
   ///		<para>The current class includes:</para>
   ///		<list type="bullet">
   ///			<item><see cref="createBackup" /> to backup VE.</item>
   ///			<item><see cref="list" /> to get list of backups.</item>
   ///          <item><see cref="restoreBackup" /> to restore backup.</item>
   ///          <item><see cref="restoreRemoteBackup" /> to restore remote backup.</item>
   ///         <item><see cref="getInfo" /> to get information about backup.</item>
   ///     </list>
   /// </remarks>
   public class Backupm 
    {
      backupmBinding backupm;
       Binder binder;

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Backupm(Binder _binder)
       {
           binder = _binder;
       }


       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Backupm(string url, string session)
       {
           binder = new Binder(url, session);
       }

      
       ///<summary>
       /// Method to backup VE
       /// <param name="ve_eid">VE eid</param>
       /// <param name="description">Backup description</param>
       /// <param name="type">Backup type
       /// <para>0-Full</para> 
       /// <para>1-Incremental</para> 
       /// <para>2-Differential</para> 
       /// </param>
       /// <param name="compression">Backup compression
       /// <para>0-no compression</para> 
       /// <para>1-normal</para>
       /// <para>2-high</para>
       /// <para>3-maximum</para>
       /// </param>
       /// <param name="ip">IP address of backup server</param>
       /// <param name="user">User name to connect to backup server</param>
       /// <param name="password">User password</param>
       /// <param name="realm">Realm</param>
       /// <param name="domain">Domain</param>
       /// <param name="protocol">Protocol</param>
       /// <param name="port">Port</param>
       /// <returns>
       /// <para>Returns backup id if backing up was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: createBackup("ac57a9c3-573b-481a-b398-d2fb0467cf4b","my backup",0,0,"10.16.3.80","root","password","00000000-0000-0000-0000-000000000000","","TCP",4433)</para>
       /// </example>
       ///</summary>
       public string createBackup(string ve_eid,string description,int type, int compression, string server_ip, string user,string password, string realm,string domain,string protocol, uint port)
       {
           try
           {
               if (backupm == null) backupm = (backupmBinding)binder.InitBinding(typeof(backupmBinding));

               backup_env backup = new backup_env();
               backup.env_list = new string[1];
               //Set VE eid
               backup.env_list[0] =ve_eid;

               backup_optionsType options = new backup_optionsType();
               options.description = System.Text.ASCIIEncoding.ASCII.GetBytes(description);
               //Allow to set backup type
               options.typeSpecified = true;
               //Set backup type
               options.type =type;
               //Allow to set backup compression
               options.compressionSpecified = true;
               //Set backup compression
               options.compression = compression;

               connection_infoType connection = new connection_infoType();
               //Set IP of backup server
               connection.address = server_ip;
               auth_nameType name = new auth_nameType();
               //Set user
               name.name = System.Text.ASCIIEncoding.ASCII.GetBytes(user);
               //Set realm
               name.realm = realm ;
               //Set domain
               name.domain = System.Text.ASCIIEncoding.ASCII.GetBytes(domain);
               //Allow to set port
               connection.portSpecified = true;
               //Set port
               connection.port = port;
               //Set protocol
               connection.protocol = protocol ;
               connection.login = name;
               //Set password
               connection.password = System.Text.ASCIIEncoding.ASCII.GetBytes(password);
               backup.backup_server = connection;

               backup.backup_options = options;

               backupm.Timeout = -1;
               return backupm.backup_env(backup)[0].id;

           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       

       ///<summary>
       /// Method to get list of backups
       /// <returns>
       /// <para>Returns list of backups and information about it.</para>
       /// </returns>
       /// <example>
       /// <para>Example: list()</para>
       /// </example>
       ///</summary>
       public string list(string backm)  
       {
           string list_result = "";
           try
           {
               if (backupm == null) backupm = (backupmBinding)binder.InitBinding(typeof(backupmBinding));
               
               list6 list =new list6 ();
               backupType[] backups = backupm.list(list);
               if (backups.Length != 0)
               {
                   foreach (backupType backup in backups)
                   {
                       if (backup.description != null)
                           list_result += "\nDescription: " + System.Text.ASCIIEncoding.ASCII.GetString(backup.description);
                       list_result += "\nID: " + backup.id +   
                       "\nEID: " + backup.eid+
                       "\nCount: " + backup.count.ToString()+
                       "\nSize: " + backup.size.ToString()+   
                       "\nType: " + backup.type.ToString()+
                       "\nInfo name: " + backup.info.name+
                       "\nTime: " + backup.time.ToString();
                   }
               }
               else
               {
                  list_result +="\nNo available backups!";
               }
               return list_result;
           
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

       }


       ///<summary>
       /// Method to restore backup
       /// <returns>
       /// <para>Returns "OK" if restoring was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: restoreBackup("85a2dd71-9133-7c44-8521-f6bd517f17ca/0")</para>
       /// </example>
       ///</summary>
       public string restoreBackup(string backup_id)
       {
           try
           {
               if (backupm == null) backupm = (backupmBinding)binder.InitBinding(typeof(backupmBinding));

               restore_env restore=new restore_env ();
               //Set backup id
               restore.backup_id = backup_id;
               backupm.restore_env(restore);
               
               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to restore remote backup
       /// <returns>
       /// <para>Returns "OK" if restoring was successful or "Exception" if not</para>
       /// </returns>
       /// <example>
       /// <para>Example: restoreBackup("85a2dd71-9133-7c44-8521-f6bd517f17ca/0","10.16.3.80","root","password","00000000-0000-0000-0000-000000000000","","TCP",4433)</para>
       /// </example>
       ///</summary>
       public string restoreRemoteBackup(string backup_id, string server_ip, string user, string realm, string domain, uint port, string protocol, string password)
       {
           try
           {
               if (backupm == null) backupm = (backupmBinding)binder.InitBinding(typeof(backupmBinding));

               restore_env restore = new restore_env();
               //Set backup id
               restore.backup_id = backup_id;

               connection_infoType connection = new connection_infoType();
               //Set IP of backup server
               connection.address = server_ip;
               auth_nameType name = new auth_nameType();
               //Set user
               name.name = System.Text.ASCIIEncoding.ASCII.GetBytes(user);
               //Set realm
               name.realm = realm;
               //Set domain
               name.domain = System.Text.ASCIIEncoding.ASCII.GetBytes(domain);
               //Allow to set port
               connection.portSpecified = true;
               //Set port
               connection.port = port;
               //Set protocol
               connection.protocol = protocol;
               connection.login = name;
               //Set password
               connection.password = System.Text.ASCIIEncoding.ASCII.GetBytes(password);
               restore.backup_server = connection;

               backupm.Timeout = -1; 

               backupm.restore_env(restore);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }

       }


       ///<summary>
       /// Method to get information about backup
       /// <returns>
       /// <para>Returns information about specified backup.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getInfo("85a2dd71-9133-7c44-8521-f6bd517f17ca/0")</para>
       /// </example>
       ///</summary>
       public string getInfo(string backup_id)
       {
           string info_result = "";
           try
           {
               if (backupm == null) backupm = (backupmBinding)binder.InitBinding(typeof(backupmBinding));

               get_info3 info = new get_info3();
               //Set backup id
               info.backup_id = backup_id;
               
               get_env_info_optionsType options = new get_env_info_optionsType();
               options.env = new get_env_info_optionsTypeEnv();
               options.excludes = new object();
               info.options = options;
               //Get data about backup
               env_backup_dataType data = (env_backup_dataType)backupm.get_info(info).info;
               info_result += "\nEnvironment info:" +
               "\n  Eid: " + data.env.eid +
               "\n  Parent eid: " + data.env.parent_eid +
               "\n  Name: " + data.env.virtual_config.name;
               if (data.env.virtual_config.description != null && data.env.virtual_config.description.Length!=0 )
                    info_result += "\n  Description: " + System.Text.ASCIIEncoding.ASCII.GetString(data.env.virtual_config.description);
               info_result += "\n  Type: " + data.env.virtual_config.type +
               "\n  Status: " + data.env.status.state.ToString()+
               "\n  Architecture: " + data.env.virtual_config.architecture+
               "\n  OS name: " + data.env.virtual_config.os.name+
               "\n  OS platform: " + data.env.virtual_config.os.platform+
               "\n  OS version: " + data.env.virtual_config.os.version+
               "\n  OS kernel: " + data.env.virtual_config.os.kernel;
               if (data.env.virtual_config.address  != null)
                   info_result += "\n  IP: " + data.env.virtual_config.address[0].ip;
               else
                   info_result += "\n  No IP address!";
               if (data.env.virtual_config.hostname != null)
                   info_result += "\n  Hostname: " + data.env.virtual_config.hostname;
               if (data.env.virtual_config.base_sample_id != null)
                   info_result += "\n  Base sample id: " + data.env.virtual_config.base_sample_id;
               if (data.env.virtual_config.base_snapshot_id != null)
                   info_result += "\n  Base snapshot id: " + data.env.virtual_config.base_snapshot_id;

               return info_result;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

    }
}
