using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 


namespace SOAPClasses
{
  public class Connect
    {
      sessionmBinding sessionm;

      public Connect()
      {
 
      }
        ///<summary>
        /// Method to create new session
        /// <param name="url">URL</param>
        /// <param name="user">User name</param>
        /// <param name="domain">Domain</param>
        /// <param name="realm">Realm</param>
        /// <param name="password">Password</param>
        /// <returns>
        /// <para>Returns session ID if OK or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: Login("http://10.16.3.19:8080","root","","00000000-0000-0000-0000-000000000000","password")</para>
        /// </example>
        ///</summary>
        public string Login(string url, string user, string domain, string realm, string password)
        {
            return LoginEx(url, user, domain, realm, password, -1);
        }

        public string LoginEx(string url, string user, string domain, string realm, string password, int timeout)
        {
           
            try
            {
                System.Net.ServicePointManager.CertificatePolicy = new TrustAllCertificatePolicy();
  
                login1 loginCmd = new login1();
                sessionm = new sessionmBinding();
                // Set SVE or HWNode(if proxy) address
                sessionm.Url = url;
                if(timeout != 0)
                    sessionm.Timeout = timeout;

                System.Text.Encoding ascii = System.Text.Encoding.ASCII;
                //Set user name
                loginCmd.name = ascii.GetBytes(user); 
                //Set domain
                if (domain.Length != 0)
                {
                    loginCmd.domain = ascii.GetBytes(domain);
                }
                //Set realm
                if (realm.Length != 0)
                {
                    loginCmd.realm = realm;
                }
                //Set user password
                loginCmd.password = ascii.GetBytes(password); 
                return sessionm.login(loginCmd).session_id;
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Login failed: " + e.Detail.InnerText);
            }
           
        }

      ///<summary>
      /// Method to close connection
      /// <param name="sessionId">sessionId</param>
      /// <param name="url">URL</param>
      /// <returns>
      /// <para>Returns "OK" if session was closed, "Exception" otherwise.</para>
      /// </returns>
      ///</summary>
      public string Logout(string sessionId, string url)
      {
          return LogoutEx(sessionId, url, -1);
      }

      public string LogoutEx(string sessionId, string url, int timeout)
      {
          try
          {
              Binder binder = new Binder(url, sessionId);
              sessionm =(sessionmBinding)binder.InitBinding(typeof(sessionmBinding));
              logout logoutCmd = new logout();
              logoutCmd.session_id = sessionId;
                          
              sessionm.logout(logoutCmd);

              return "OK";
          }
          catch (System.Web.Services.Protocols.SoapException e)
          {
              throw new ApplicationException("Logout failed: " + e.Detail.InnerText);
          }
      }

        ///<summary>
        /// CertificatePolicy class
        ///</summary>
        private class TrustAllCertificatePolicy : System.Net.ICertificatePolicy
        {
            public TrustAllCertificatePolicy()
            { }

            public bool CheckValidationResult(System.Net.ServicePoint sp,
                System.Security.Cryptography.X509Certificates.X509Certificate cert,
                System.Net.WebRequest req, int problem)
            {
                return true;
            }
        }
    }
}
