using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA;

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with envm.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="createVE" /> to create VE.</item>
    ///			<item><see cref="getVEList" /> to get VE list.</item>
    ///         <item><see cref="getVEState" /> to lget VE status.</item>
    ///         <item><see cref="startVE" /> to start VE.</item>
    ///         <item><see cref="stopVE" /> to stop VE.</item>
    ///         <item><see cref="restartVE" /> to restart VE.</item>
    ///         <item><see cref="suspendVE" /> to suspend VE.</item>
    ///         <item><see cref="resumeVE" /> to resume VE.</item>
    ///         <item><see cref="destroyVE" /> to destroy VE.</item>    
    ///         <item><see cref="modifyIP" /> to modify VE IP.</item>
    ///         <item><see cref="modifyHostname" /> to modify VE hostname.</item>
    ///         <item><see cref="modifyName" /> to modify VE name.</item>
    ///         <item><see cref="modifyDNS" /> to modify VE DNS.</item>
    ///         <item><see cref="modifyDescription" /> to modify VE description.</item>
    ///         <item><see cref="modifyOfflineManagement" /> to modify VE offline management.</item>
    ///         <item><see cref="modifyQoS" /> to modify VE QoS.</item>
    ///         <item><see cref="getConfig" /> to get VE configuration.</item>
    ///         <item><see cref="nameToEid" /> to get VE EID from VE name.</item>
    ///     </list>
    /// </remarks>
   public class Envm 
    {
        vzaenvmBinding env ;
        Binder binder;

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
       public Envm(Binder _binder)
       {
           binder = _binder;
       }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public Envm(string url,string session)
        {
            binder = new Binder(url, session);
            
        }


        ///<summary>
        ///  Method to create VE
        /// <param name="name">VE name</param>
        /// <param name="os_template">OS template for VE</param>
        /// <param name="platform">OS platform</param>
        /// <param name="architecture">Architecture</param>
        /// <param name="hostname">VE hostname</param>
        /// <param name="ip">VE ip address</param>
        /// <param name="mask">VE network mask</param>
        /// <param name="network">VE network identifier
        /// <para>venet0 - for Linux</para>
        /// <para>venet1 - for Windows</para>
        /// </param>
        /// <param name="offline_managenent">Offline management</param>
        /// <returns>
        /// <para>Returns VE EID.</para>
        /// </returns>
        /// <example>
        ///  <para>Example: createVE("sample_ve","redhat-as3","linux","ia64","sample_ve_hostname","10.16.3.179","255.255.255.0","venet0",true) </para>
        /// </example>
        ///</summary>
       public string createVE(string name, string os_template, string platform, string architecture, string sampleId, string hostname, string ip, string netmask, string network, bool offline_management)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding), true);   

                create create = new create();
                
                venv_configType1 veconfig = new venv_configType1();                 
                //Set OS template
                templateType osTemplate = new templateType();
                osTemplate.name = os_template;
                veconfig.os_template = osTemplate; 
                //Set base sample
                veconfig.base_sample_id = sampleId;
                veconfig.os = new osType();
                //Set platform
                veconfig.os.platform = platform;  
                //Set type
                veconfig.type = "virtuozzo";
                //Set VE name
                veconfig.name = name;
                //Set architecture
                veconfig.architecture = architecture;
                //Set VE hostname
                veconfig.hostname = hostname; 
                //Set IP address and netmask
                ip_addressType[] ip_address = new ip_addressType[1];
                ip_address[0] = new ip_addressType();
                ip_address[0].ip = ip;
                ip_address[0].netmask = netmask;

                net_vethType[] net = new net_vethType[1];
                net[0] = new net_vethType(); 
                //Set network
                net[0].host_routed = new object();
                net[0].id = network;
                net[0].ip_address = ip_address;
                veconfig.net_device = net; 
                //Set offline management
                veconfig.offline_managementSpecified = true;
                veconfig.offline_management = offline_management; 

                //Set all generated configuration 
                create.config = veconfig;

                //Create VE  
                return env.create(create).env.eid;

            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to create CT '" + name + "' - " + e.Detail.InnerText);
            }
        }

        ///<summary>
        ///  Method to get VE samples 
        /// <param name="platform">OS platform</param>
        /// <param name="architecture">Architecture</param>
        /// <returns>
        /// <para>Returns sample id.</para>
        /// </returns>
        /// <example>
        ///  <para>Example: getSample("linux","ia64") </para>
        /// </example>
        ///</summary>
       public string getSample(string platform, string architecture, string name)
       {
           try
           {
               env_samplemBinding env_sample = (env_samplemBinding)binder.InitBinding(typeof(env_samplemBinding));
               get_sample_conf get_sample = new get_sample_conf();
               sample_confType[] samples = env_sample.get_sample_conf(get_sample);
               
               if (samples != null)
               {
                   foreach (sample_confType sample in samples)
                   {
                       if (sample.env_config.os != null)
                       {
                           if ((platform.Length == 0 || sample.env_config.os.platform == platform) && (architecture.Length ==0 || sample.env_config.architecture == architecture) && (name.Length == 0 || sample.name == name))
                           {
                               return sample.id + " " + sample.name + " " + sample.env_config.os.platform + " " + sample.env_config.architecture + "\n";
                           }
                       }
                   }
               }

               return "";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       public string getSamples()
       {
           try
           {
               env_samplemBinding env_sample = (env_samplemBinding)binder.InitBinding(typeof(env_samplemBinding));
               get_sample_conf get_sample = new get_sample_conf();
               sample_confType[] samples = env_sample.get_sample_conf(get_sample);
               string ret = "";

               if (samples != null)
               {
                   foreach (sample_confType sample in samples)
                   {
                       ret += sample.id;
                       if (sample.name != null)
                           ret += " " + sample.name;

                       if (sample.env_config != null)
                       {
                            if (sample.env_config.os != null && sample.env_config.os.platform != null)
                                ret += " " + sample.env_config.os.platform;
                            if (sample.env_config.architecture != null)
                                ret += " " + sample.env_config.architecture;
                       }
                       ret += "\n";
                   }
               }

               return ret;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       public string getVEStateName(int state)
       {
           switch (state)
           {
               case 1: 
                   return "unexist";
               case 3: 
                   return "down";
               case 4: 
                   return "mounted";
               case 5: 
                   return "suspended";
               case 6: 
                   return "running";
               case 7: 
                   return "repairing";
           }

           return "unknown";
       }

       public string getVEListEx()
       {
           if (env == null) 
               env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

           get_list1 velist = new get_list1();

           string ret = "";

           try
           {
               foreach (string venv in env.get_list(velist))
               {
                   string state;
                   try
                   {
                       state = getVEState(venv);
                   }
                   catch (System.Web.Services.Protocols.SoapException e)
                   {
                       throw new ApplicationException("Failed to get CT '" + venv + "' state - " + e.Detail.InnerText);
                   }

                   int veid;
                   try
                   {
                       veid = getVEID(venv);
                   }
                   catch (System.Web.Services.Protocols.SoapException e)
                   {
                       throw new ApplicationException("Failed to get CT '" + venv + "' ID - " + e.Detail.InnerText);
                   }

                   ret += "#" + veid + " " + venv + " " + getVEStateName(System.Convert.ToInt16(state)) + "\n";
               }
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               throw new ApplicationException("Failed to get CT list - " + e.Detail.InnerText);
           }

           return ret;
       }


        ///<summary>
        /// Method to get VE list
        /// <param name="status">VE status
        /// <para >1-unexist</para>
        /// <para >3-down</para>
        /// <para >4-mounted</para>
        /// <para >5-suspended</para>
        /// <para >6-running</para>
        /// <para >7-repairing</para>
        /// </param>
        /// <returns>
        /// <para>Returns VE list.</para>
        /// </returns>
        /// <example>
        /// <para>Example: getVEList(6)</para>
        /// </example>
        ///</summary>
        public string getVEList(int status)
        {
            string list_result = "";
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));
                
                get_list1 velist = new get_list1();
                env_statusType[] env_status = new env_statusType[1];
                env_status[0] = new env_statusType();

                //Set state
                env_status[0].state = status;
                velist.status = env_status;

                list_result += "List of VE in state="+status .ToString ()+": \n";
                foreach (string venv in env.get_list(velist))
                {
                    list_result += venv + "\n";
                }
                return list_result;

            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

       public eidList getEIDList(int status)
       {
           eidList list_result = new eidList();
           if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

           get_list1 velist = new get_list1();
           env_statusType[] env_status = new env_statusType[1];
           env_status[0] = new env_statusType();

           //Set state
           env_status[0].state = status;
           velist.status = env_status;

           foreach (string venv in env.get_list(velist))
           {
               list_result.Add(venv);
           }
           return list_result;
       }

       public int getVEID(string eid)
       {
           if (env == null) 
               env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));
           get_info2 args = new get_info2();
           args.eid = new string[1] { eid };
           envType[] envs = env.get_info(args);
           foreach (envType e in envs)
           {
               venv_configType1 vzaconf = e.virtual_config as venv_configType1;
               return (int)vzaconf.veid;
           }
           throw new System.Exception("can't get VEID by eid for EID '" + eid + "'");
       }


        ///<summary>
        /// Method to get VE status
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns VE state.</para>
        /// </returns>
        /// <example>
        /// <para>Example: getVEState("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getVEState(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));
                
                get_info2 get = new get_info2();
                //Set VE eid
                get.eid = new string[1];
                get.eid[0] = ve_eid;

                return env.get_info(get)[0].status.state.ToString ();
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to get CT '" + ve_eid + "' state - " + e.Detail.InnerText);
            }
        }


        ///<summary>
        /// Method to search specified VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was found or "Exception" if not .</para>
        /// </returns>
        /// <example>
        /// <para>Example: searchVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
       public string searchVE(string ve_eid)
       {
           try
           {
               if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

               get_list1 velist = new get_list1();

               string[] ves = env.get_list(velist);

               foreach (string venv in ves)
               {
                   if (ve_eid == venv)
                       return "OK!";
               }
               return "Error: No such VE!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
 
       }

        #region VE actions(start, stop, restart, destroy, resume , suspend)

        ///<summary>
        /// Method to start VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was started or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: startVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string startVE(string ve_eid)
        {                                                             
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                start startve = new start();
                //Set VE eid
                startve.eid = ve_eid;
                env.start(startve);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to start CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }
        }

        ///<summary>
        /// Method to stop VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was stopped or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: stopVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string stopVE(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                stop1 stopve = new stop1();
                //Set VE eid
                stopve.eid = ve_eid;
                env.stop(stopve);

                return "OK";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to stop CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }   
        }

        ///<summary>
        /// Method to restart VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was restarted or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: restartVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string restartVE(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                restart restartve = new restart();
                //Set VE eid
                restartve.eid = ve_eid;
                env.restart(restartve);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to restart CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }   
        }

        ///<summary>
        /// Method to destroy VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was destroyed or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: destroyVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string destroyVE(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                destroy destroyve = new destroy();
                //Set VE eid
                destroyve.eid = ve_eid;
                env.destroy(destroyve);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to destroy CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }  
        }

        ///<summary>
        /// Method to suspend VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was suspended or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: suspendVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string suspendVE(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                suspend1  suspendve = new suspend1 ();
                //Set VE eid
                suspendve.eid = ve_eid;
                env.suspend(suspendve);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to suspend CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }   
        }

        ///<summary>
        /// Method to resume VE
        /// <param name="ve_eid">VE eid</param>
        /// <returns>
        /// <para>Returns "OK" if VE was resumed or "Exception" if not </para>
        /// </returns>
        /// <example>
        /// <para>Example: resumeVE("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string resumeVE(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                resume1 resumeve = new resume1();
                //Set VE eid
                resumeve.eid = ve_eid;
                env.resume(resumeve);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                throw new ApplicationException("Failed to resume CT '" + ve_eid + "' - " + e.Detail.InnerText);
            }  
        }

#endregion

        
        #region Modify VE configuration

        ///<summary>
        /// Method to modify IP address
        /// <param name="ve_eid">VE eid</param>
        /// <param name="ip">IP address</param>
        /// <param name="netmask">Network mask</param>
        /// <param name="network">Network id</param>
        /// <returns><para>Returns "OK" if VE ip was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyIP("ac57a9c3-573b-481a-b398-d2fb0467cf4b","10.16.3.170","255.255.255.0","venet0")</para>
        /// </example>
        ///</summary>
        public string modifyIP(string ve_eid,string new_ip, string netmask,string network)
        {
          try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 m_set = new set2();
                //Set VE eid  
                m_set.eid = ve_eid;
            
                venv_configType1 m_veconfig = new venv_configType1();                
                //Set ip addresses
                ip_addressType[] ip_address = new ip_addressType[1];
                ip_address[0] = new ip_addressType();
                ip_address[0].ip = new_ip;
                ip_address[0].netmask = netmask;

                net_vethType[] m_net = new net_vethType[1];
                m_net[0] = new net_vethType();
                //Set network
                m_net[0].host_routed = new object();
                m_net[0].id = network;
                m_net[0].ip_address = ip_address;
                m_veconfig.net_device = m_net;
                m_set.config = m_veconfig; 
                env.set(m_set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            } 
            
        }

        ///<summary>
        /// Method to modify IP address
        /// <param name="ve_eid">VE eid</param>
        /// <param name="ip">IP address</param>
        /// <param name="netmask">Network mask</param>
        /// <param name="network">Network id</param>
        /// <returns><para>Returns "OK" if VE ip was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyIP("ac57a9c3-573b-481a-b398-d2fb0467cf4b","10.16.3.170","255.255.255.0","venet0")</para>
        /// </example>
        ///</summary>
        public string modifyCapabilities(string ve_eid, string id, bool value)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 m_set = new set2();
                //Set VE eid  
                m_set.eid = ve_eid;

                venv_configType1 m_veconfig = new venv_configType1();
                //Set ip addresses
                m_veconfig.capability = new venv_configTypeCapability[1];
                m_veconfig.capability[0] = new venv_configTypeCapability();
                m_veconfig.capability[0].id = "vps_id_label";
                m_veconfig.capability[0].value = value;

                
                m_set.config = m_veconfig;
                env.set(m_set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify hostname
        /// <param name="ve_eid">VE eid</param>
        /// <param name="hostname">New VE hostname</param>
        /// <returns><para>Returns "OK" if VE hostname was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyHostname("ac57a9c3-573b-481a-b398-d2fb0467cf4b","sample_ve_updated")</para>
        /// </example>
        ///</summary>
        public string modifyHostname(string ve_eid, string new_hostname)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 m_set = new set2();
                //Set VE eid
                m_set.eid = ve_eid;

                venv_configType1 m_veconf = new venv_configType1();
                //Set new hostname
                m_veconf.hostname = new_hostname;
                m_set.config = m_veconf;
                env.set(m_set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify name
        /// <param name="ve_eid">VE eid</param>
        /// <param name="name">New VE name</param>
        /// <returns><para>Returns "OK" if VE name was modified or "Exception" if not</para>  </returns>
        /// <example>
        /// <para>Example: modifyName("ac57a9c3-573b-481a-b398-d2fb0467cf4b","sample_ve_modified")</para>
        /// </example>
        ///</summary>
        public string modifyName(string ve_eid,string new_name)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 set = new set2();
                //Set VE eid
                set.eid = ve_eid;
                venv_configType1 veconf = new venv_configType1();
                //Set new VE name
                veconf.name = new_name;
                set.config = veconf;
                env.set(set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify offline management
        /// <param name="ve_eid">VE eid</param>
        /// <param name="managenent_activate">"true"-activate offline management, "false"-disactivate</param>
        /// <returns><para>Returns "OK" if offline management was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyOfflineManagement("ac57a9c3-573b-481a-b398-d2fb0467cf4b",false)</para>
        /// </example>
        ///</summary>
        public string modifyOfflineManagement(string ve_eid,bool managenent_activate)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 set = new set2();
                //Set VE eid
                set.eid = ve_eid;
                venv_configType1 veconfig = new venv_configType1(); 
                //Allow to specify offline management
                veconfig.offline_managementSpecified = true;        
                //Set offline management
                veconfig.offline_management = managenent_activate;                 
                set.config = veconfig;
                env.set(set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify VE description
        /// <param name="ve_eid">VE eid</param>
        /// <param name="description">VE description</param>
        /// <returns><para>Returns "OK" if description was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyDescription("ac57a9c3-573b-481a-b398-d2fb0467cf4b","my sample VE")</para>
        /// </example>
        ///</summary>
        public string modifyDescription(string ve_eid, string new_description)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 set = new set2();
                //Set VE eid
                set.eid = ve_eid;
                venv_configType1 veconfig = new venv_configType1();
                //Set VE description
                veconfig.description = System.Text.Encoding.ASCII.GetBytes(new_description);
                set.config = veconfig;
                env.set(set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify VE QoS
        /// <param name="ve_eid">VE eid</param>
        /// <param name="qos_id">QoS id: 
        /// <para>diskinodes</para>
        /// <para>diskspace</para>
        /// <para>numproc</para>
        /// <para>numtcpsock</para>
        /// <para>cpulimit</para>
        /// <para>cpus</para>  
        /// <para>etc.</para>
        /// </param>
        /// <param name="hard">Hard limit</param>
        /// <param name="soft">Soft limit</param>
        /// <returns><para>Returns "OK" if QoS was modified or "Exception" if not</para> </returns>
        /// <example>
        /// <para>Example: modifyQoS("ac57a9c3-573b-481a-b398-d2fb0467cf4b","diskinodes",10000,10000)</para>
        /// </example>
        ///</summary>
        public string modifyQoS(string ve_eid, string qos_id, int hard,int soft)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 set = new set2();
                //Set VE eid
                set.eid = ve_eid;

                venv_configType1 veconfig = new venv_configType1();
                //Set VE QoS
                veconfig.qos=new qosType[1];
                veconfig.qos[0]=new qosType ();
                //Set QoS ID
                veconfig.qos[0].id = qos_id;
                //Allow to set hard limit
                veconfig.qos[0].hardSpecified = true;
                //Set hard limit
                veconfig.qos[0].hard = hard;
                //Allow to set soft limit
                veconfig.qos[0].softSpecified = true;
                //Set soft limit
                veconfig.qos[0].soft = soft;
                
                set.config = veconfig;
                env.set(set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

        ///<summary>
        /// Method to modify VE DNS servers
        /// <param name="ve_eid">VE eid</param>
        /// <param name="nameserver">DNS</param>
        /// <returns><para>Returns "OK" if DNS was modified or "Exception" if not </para> </returns>
        /// <example>
        /// <para>Example: modifyDNS("ac57a9c3-573b-481a-b398-d2fb0467cf4b","my.ve.hosts.ru")</para>
        /// </example>
        ///</summary>
        public string modifyDNS(string ve_eid, string new_nameserver)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                set2 set = new set2();
                //Set VE eid
                set.eid = ve_eid;

                venv_configType1 veconfig = new venv_configType1();  
                veconfig.net_device=new net_vethType [1];
                veconfig.net_device[0]=new net_vethType ();
                //Set VE DNS
                veconfig.net_device[0].nameserver=new string [1];
                veconfig.net_device[0].nameserver[0]=new_nameserver;
                
                set.config = veconfig;
                env.set(set);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }

        }

       public string setUserPassword(string ve_eid, string name, byte [] password)
       {
           try
           {
               if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

               set_user_password set = new set_user_password();

               set.eid = ve_eid;
               set.name = name;
               set.password = password;

               env.set_user_password(set);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }

       }

        #endregion


        #region Get VE configuration

        ///<summary>
        /// Method to get VE configuration
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE configuration</para> </returns>
        /// <example>
        /// <para>Example: getConfig("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getConfig(string ve_eid)
        {
            string ve_info = "";
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config  ;
                //Get Name
                ve_info += "Name: " + envtype[0].virtual_config.name + "\n"; 
                //Get Description
                if (envtype[0].virtual_config.description!=null && envtype[0].virtual_config.description.Length != 0)
                    ve_info += "Description: " + System.Text.Encoding.ASCII.GetString(envtype[0].virtual_config.description) + "\n" + 
                //Get network configaration
                "Network configuration: \n";
                if (envtype[0].virtual_config.address != null)
                {
                    ve_info +="IP: " + veconfig.address[0].ip+"\n"+
                    "Netmask: " + veconfig.address[0].netmask+"\n";
                }
                ve_info += "HostName: " + veconfig.hostname + "\n" +
                    //Get architecture
                "Architecture: " + veconfig.architecture + "\n" +
                    //Get OS
                "OS name: " + veconfig.os.name + "\n" +
                "OS platform: " + veconfig.os.platform + "\n" +
                "OS kernel: " + veconfig.os.kernel + "\n" +
                "OS version: " + veconfig.os.version + "\n" +
                    //Get status 
                "Status: " + envtype[0].status.state.ToString() + "\n" +
                    //Get QoS
                "QoS cur: " + veconfig.qos[0].cur.ToString() + "\n" +
                "QoS hard: " + veconfig.qos[0].hard.ToString() + "\n" +
                "QoS id: " + veconfig.qos[0].id + "\n" +
                "QoS soft: " + veconfig.qos[0].soft.ToString();// +"\n" +
                    //Get offline Management                                 
                //"Offline management specified: " + veconfig.offline_managementSpecified.ToString() + "\n" +
                //"Offline management: " + veconfig.offline_management.ToString() + "\n";
                return ve_info; 
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }


        ///<summary>
        /// Method to get VE offline management
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE offline management status</para> </returns>
        /// <example>
        /// <para>Example: getOfflineManagement("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        //public string getOfflineManagement(string ve_eid)
        //{
        //    try
        //    {
        //        if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

        //        get_info2 getInfo = new get_info2();
        //        string[] eids = new string[1];
        //        //Set VE eid
        //        eids[0] = ve_eid;
        //        getInfo.eid = eids;

        //        envType[] envtype = env.get_info(getInfo);
        //        venv_configType veconfig = envtype[0].virtual_config;
               
        //        //Get offline Management                                 
        //        if (envtype[0].config. )
        //            return veconfig.offline_management.ToString();
        //        else return "Offline management is not specified!";
        //    }
        //    catch (System.Web.Services.Protocols.SoapException e)
        //    {
        //        return "Exception: " + e.Detail.InnerText;
        //    }
        //}



        ///<summary>
        /// Method to get VE description
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE description</para> </returns>
        /// <example>
        /// <para>Example: getDescription("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getDescription(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config;
                //Get Description
                if (envtype[0].virtual_config.description != null && envtype[0].virtual_config.description.Length != 0)
                    return System.Text.Encoding.ASCII.GetString(envtype[0].virtual_config.description);
                else
                    return "No description!";
                       
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        ///<summary>
        /// Method to get VE IP
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE IP</para> </returns>
        /// <example>
        /// <para>Example: getIP("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getIP(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config;
                if (envtype[0].virtual_config.address != null)
                    return veconfig.address[0].ip;
                else return "No IP!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        ///<summary>
        /// Method to get VE hostname
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE hostname</para> </returns>
        /// <example>
        /// <para>Example: getHostname("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getHostname(string ve_eid)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config;
                //Get Hostname
                return veconfig.hostname;
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        ///<summary>
        /// Method to get VE QoS
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE QoS</para> </returns>
        /// <example>
        /// <para>Example: getQoS("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getQoS(string ve_eid,string qos)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config;
                //Get QoS
                for (int i = 0; i < veconfig.qos.Length; i++)
                {
                    if (veconfig.qos[i].id == qos)
                          return  veconfig.qos[i].hard.ToString();
                }
                return "No such qos!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }



        ///<summary>
        /// Method to get VE name
        /// <param name="ve_eid">VE eid</param>
        /// <returns><para>Returns VE name</para> </returns>
        /// <example>
        /// <para>Example: getName("ac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
        /// </example>
        ///</summary>
        public string getName(string ve_eid)
        {
           try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                string[] eids = new string[1];
                //Set VE eid
                eids[0] = ve_eid;
                getInfo.eid = eids;

                envType[] envtype = env.get_info(getInfo);
                venv_configType veconfig = envtype[0].virtual_config;
                //Get Name
                return envtype[0].virtual_config.name;
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }
        #endregion

        ///<summary>
        /// Method to get VE EID from VE name
        /// <param name="name">VE name</param>
        /// <returns >
        /// <para>Returns VE eid </para>
        /// </returns>
        /// <example>
        /// <para>Example: nameToEid("sample_ve")</para>
        /// </example>
        ///</summary>
        public string nameToEid(string name)
        {
            try
            {
                if (env == null) env = (vzaenvmBinding)binder.InitBinding(typeof(vzaenvmBinding));

                get_info2 getInfo = new get_info2();
                getInfo.eid = new string[1];
                get_list1 velist = new get_list1();
                string eids = "";
                string[] nn = env.get_list(velist);
                foreach (string eid in nn)
                {
                    getInfo.eid[0] = eid;
                    envType[] envs = env.get_info(getInfo);
                    if (envs.Length != 0)
                    {
                        if (env.get_info(getInfo)[0].virtual_config.name == name)
                        {
                            eids = eid;
                            break;
                        }
                        else
                        {
                            eids = "";
                        }
                    }

                }

                return eids;
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }
    }
}
