using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 
using System.IO;
using System.Threading;
 

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with filer.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="uploadFile" /> to upload specified file.</item>
    ///         <item><see cref="uploadBigFile" /> to upload specified big file.</item>
    ///			<item><see cref="downloadFile" /> to download specified file.</item>
    ///     </list>
    /// </remarks>
    public class Filer
    {
        filerBinding filer;
        Binder binder;

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public Filer(Binder _binder)
        {
            binder = _binder;                     
        }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public Filer(string url, string session)
        {
            binder = new Binder(url, session);
            
        }

        ///<summary>
        /// Method to upload file to environment
        /// <param name="source">Source file</param>
        /// <param name="destination">Destination file</param>
        /// <param name="offset">Destination file
        /// <para>-2 - Rewrite file if exists</para>
        /// <para>-1 - Add data to the end of the file</para>
        /// </param>
        /// <returns>
        /// <para>Returns "OK" if was uploaded or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: uploadFile(@"c:\share\packets.txt",@"/tmp/pack15",-2)</para>
        /// </example>
        ///</summary>
        public string uploadFile(string source,string destination,int offset)
        {
            try
            {
                if (filer == null) filer = (filerBinding)binder.InitBinding(typeof(filerBinding));
               
                upload uploader = new upload();
                uploader.file = new uploadFile[1];
                uploader.file[0] = new uploadFile();
                //If destination file exists
                uploader.force = new object (); 
                //Allow to set offset
                uploader.file[0].offsetSpecified = true;
                //Set destination file path
                uploader.file[0].path = System.Text.ASCIIEncoding.ASCII.GetBytes(destination);
                //Set maximum buffer size
                int readBlockSize = 100;
                FileInfo info = new FileInfo(source);
                long files_count = (long)info.Length / readBlockSize + 1;
                uploader.file = new uploadFile[files_count];

                if (info.Length > readBlockSize)
                {
                    FileStream fstream = new FileStream(source, FileMode.Open, FileAccess.Read);
                    int current = 0;
                    while ((fstream.Length - fstream.Position) > readBlockSize)
                    {
                        byte[] readBlock = new byte[readBlockSize];
                        fstream.Read(readBlock, 0, readBlock.Length);
                        setFileConfig(uploader, current, offset,destination, readBlock);
                        current++;
                    }
                    byte[] restData = new byte[fstream.Length - fstream.Position];
                    fstream.Read(restData, 0, restData.Length);
                    setFileConfig(uploader, current, offset, destination, restData);
                    fstream.Close();
                }
                filer.upload(uploader);
               
                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
              
        }
       

        ///<summary>
        /// Method to get files' list
        /// <param name="source">Path to get files from</param>
        /// <returns>
        /// <para>Returns files' information if was files were listed or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: list(path)</para>
        /// </example>
        ///</summary>
        public string list(string dir)
        {
            try
            {
                string list_result = "";
                if (filer == null) filer = (filerBinding)binder.InitBinding(typeof(filerBinding));
                list4 list=new list4 ();
                string[] path = new string[] { dir };
                byte[][] paths = new byte[path.Length][];

                //Set paths
                for (int i = 0; i < path.Length; i++)
                    paths[i] = System.Text.ASCIIEncoding.ASCII.GetBytes(path[i]);
                list.path = paths;
                foreach (fileType file in filer.list(list))
                {
                     list_result += "\n\nName: " + System.Text.ASCIIEncoding.ASCII.GetString(file.name)+
                         "\nSize: " + file.size.ToString() + "\nType: " + file.type.ToString (); 
                }

                return list_result;
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        ///<summary>
        /// Method to upload big file to environment
        /// <param name="source">Source file</param>
        /// <param name="destination">Destination file</param>
        /// <returns>
        /// <para>Returns "OK" if was uploaded or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: uploadFile(@"c:\share\packets.txt",@"/tmp/pack15")</para>
        /// </example>
        ///</summary>
        public string uploadBigFile(string b_source, string b_destination)
        {
            try
            {
                if (filer == null) filer = (filerBinding)binder.InitBinding(typeof(filerBinding));

                //Add data to the end of the file
                int offset = -1;
                //Set maximum buffer size
                long readBlockSize = 524288;
                upload uploader = new upload();
                FileInfo info = new FileInfo(b_source);
                long files_count= (long)info.Length/readBlockSize +1;
                uploader.file = new uploadFile[files_count];
                
                if (info.Length > readBlockSize)
                {
                    FileStream fstream = new FileStream(b_source, FileMode.Open, FileAccess.Read);
                   int current = 0;
                   while ((fstream.Length - fstream.Position) > readBlockSize)
                   {
                       byte[] readBlock = new byte[readBlockSize];
                       fstream.Read(readBlock, 0, readBlock.Length);
                       setFileConfig(uploader, current, offset, b_destination, readBlock);  
                       current++;
                   }
                   byte[] restData = new byte[fstream.Length - fstream.Position];
                   fstream.Read(restData, 0, restData.Length);
                   setFileConfig(uploader, current, offset, b_destination, restData);
                   fstream.Close();
                }
                filer.upload(uploader);

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        void setFileConfig(upload uploader,int current, int offset, string destination, byte[] body)
        {
            uploader.file[current] = new uploadFile();
            //If destination file exists
            uploader.force = new object();
            //Allow to set offset
            uploader.file[current].offsetSpecified = true;
            //Set offset
            uploader.file[current].offset = offset;
            //Set destination file path
            uploader.file[current].path = System.Text.ASCIIEncoding.ASCII.GetBytes(destination);
            uploader.file[current].body = body;
            uploader.file[current].size = body.Length;
        }


        ///<summary>
        /// Method to download file from environment
        /// <param name="source">Source file</param>
        /// <param name="destination">Destination file</param>
        /// <param name="offset">Destination file
        /// <para>-2 - Rewrite file if exists</para>
        /// <para>-1 - Add data to the end of the file</para>
        /// </param>
        /// <returns>
        /// <para>Returns "OK" if file was downloaded or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: downloadFile(@"/tmp/pack15",@"c:\share\result",-2)</para>
        /// </example>
        ///</summary>
        public string downloadFile(string d_source, string d_destination, int offset)
        {
            try
            {
                if (filer == null) filer = (filerBinding)binder.InitBinding(typeof(filerBinding));
               
                download downloader = new download();
                downloader.file = new downloadFile[1];
                downloader.file[0] = new downloadFile();
                //Allow to set offset
                downloader.file[0].offsetSpecified = true;
                //Set offset
                downloader.file[0].offset = offset;
                //Set path to downloadable file
                downloader.file[0].path = System.Text.ASCIIEncoding.ASCII.GetBytes(d_source);
                //Get the file body (BUFFER_SIZE=512KB)
                string filestr = System.Text.ASCIIEncoding.ASCII.GetString(filer.download(downloader)[0].body);                
                //Save it to destination file
                StreamWriter stream = new StreamWriter(d_destination);
                stream.Write(filestr);
                stream.Flush();
                stream.Close();

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

        ///<summary>
        /// Method to copy file from environment
        /// <param name="source">Source file</param>
        /// <param name="destination">Destination file</param>
        /// <returns>
        /// <para>Returns "OK" if file was downloaded or "Exception" if not.</para>
        /// </returns>
        /// <example>
        /// <para>Example: downloadFile(@"/tmp/pack15",@"c:\share\result")</para>
        /// </example>
        ///</summary>
        public string copyFile(string d_source, string d_destination, int offset)
        {
            try
            {
                if (filer == null) filer = (filerBinding)binder.InitBinding(typeof(filerBinding));

                copy copier=new copy ();
                string[] path = new string[] { d_source };                
                byte[][] paths = new byte[path.Length ][];
                for (int i = 0; i < path.Length; i++)
                   paths[i] = System.Text.ASCIIEncoding.ASCII.GetBytes(path[i]);
                copier.path = paths;
                copier.dst_path =System.Text.ASCIIEncoding.ASCII.GetBytes(d_destination);

                filer.copy(copier); 

                return "OK!";
            }
            catch (System.Web.Services.Protocols.SoapException e)
            {
                return "Exception: " + e.Detail.InnerText;
            }
        }

    }
}
