using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 
using System.Xml ;
using System.IO;
using System.Collections;

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with infrastrurem.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="addUnit" /> to add new unit.</item>
    ///			<item><see cref="removeUnit" /> to remove unit.</item>
    ///         <item><see cref="getConfig" /> to get system configaration(base units).</item>
    ///         <item><see cref="list" /> to list units.</item>
    ///         <item><see cref="addEnv" /> to add environment to unit.</item>
    ///         <item><see cref="moveEnv" /> to move environment from one unit to another.</item>
    ///     </list>
    /// </remarks>
   public class Infrastructurem
    {
       infrastructuremBinding infrastruct;
       Binder binder;

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Infrastructurem(Binder _binder)
       {
           binder = _binder;
       }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
       public Infrastructurem(string url, string session)
        {
            binder = new Binder(url, session);
            
        }

                
      ///<summary>
      /// Method to add new unit
      /// <param name="name">Unit to add</param>
      /// <returns>
       /// <para>Returns "OK" if adding was successful or "Exception" if not.</para>
      /// </returns>
      /// <example>
      /// <para>Example: addUnit("ou=QA,o=VZLOrganization,dc=vzl")</para>
      /// </example>
      ///</summary>
      public string addUnit(string name,string base_name)
       {
           try
           {
               if (infrastruct == null) infrastruct = (infrastructuremBinding)binder.InitBinding(typeof(infrastructuremBinding));
              
               add3 add = new add3();
               unit_infoType unitinfo = new unit_infoType();
               //Set unit name
               unitinfo.name = name + "," + base_name;
               add.unit = unitinfo;
               infrastruct.add(add);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       ///<summary>
       /// Method to infrastructure parameters from system configuration
       /// <returns>
       /// <para>Returns parameters if getting was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getConfig()</para>
       /// </example>
       ///</summary>
       public Hashtable getConfig()
       {
          try
           {
               systemBinding system = (systemBinding)binder.InitBinding(typeof(systemBinding),false);
               object get_config = new object();
               Hashtable hash = new Hashtable();
               XmlElement[] elems = system.get_configuration(get_config).Any;
               foreach (XmlElement elem in elems)
               {
                    if (elem.Name.Contains("infrastructure"))
                   {
                      getAttributes(elem.ChildNodes,ref hash);
                  }
               }
              
               return hash;
           }
           catch (Exception )
           {
               return null;
           }
       }

       ///<summary>
       /// Method to get base units from system configuration
       /// <returns>
       /// <para>Returns base units if getting was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getConfig()</para>
       /// </example>
       ///</summary>
       public string getBaseDN(string sn)
       {
           return getConfig()["organization_base_dn"].ToString();
       }
      

       void getAttributes(XmlNodeList config,ref Hashtable hash)
       {
         //  hash = new Hashtable();
           try
           {
              foreach (XmlNode node in config)
               {
                   if (node.Name != "#whitespace" && node.Name != "#text")
                   {
                       if (node.HasChildNodes && node.ChildNodes.Count <= 1)
                       {
                          hash.Add(node.Name, node.InnerText);
                       }
                       else
                       {
                           getAttributes(node.ChildNodes,ref hash);
                       }
                   }
               }
               
           }
           catch (Exception e)
           {
               throw e;
           }

       }
       
      

       ///<summary>
       /// Method to remove unit
       /// <param name="name">Unit to remove</param>
       /// <returns>
       /// <para>Returns "OK" if removing was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: removeUnit("ou=SoftDev,o=VZLOrganization,dc=vzl")</para>
       /// </example>
       ///</summary>
       public string removeUnit(string name,string base_name)
       {
           try
           {
               if (infrastruct == null) infrastruct = (infrastructuremBinding)binder.InitBinding(typeof(infrastructuremBinding));
               
               del2 del = new del2();
               //Set unit name
               del.name = name + "," + base_name;
               infrastruct.del(del);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }
            
       ///<summary>
       /// Method to list units
       /// <param name="name"><para>Unit to list</para></param>
       /// <param name="scope" type="integer">
	   ///     <para>Number of subtrees to list. </para>
       ///     <para> 0 - return the information about the specified object only</para>
       ///     <para> 1 - return the information for the direct children of the specified object only </para>
       ///     <para> 2 - return the information for the specified object and the entire sub-tree under the specified object's DN </para>
       /// </param>
       /// <returns type="string">
       /// <para>Returns the list of units.</para>
       /// </returns>
       /// <example>
       ///  <para>Example: list("o=VZLOrganization,dc=vzl",2) </para>
       /// </example>
       ///</summary>
       public string list(string base_name, int scope)
       {
           string list_result = "";
           try
           {
               if (infrastruct == null) infrastruct = (infrastructuremBinding)binder.InitBinding(typeof(infrastructuremBinding));

               list6 list=new list6 ();
               //Set unit name
               list.name = base_name;
               //Enable "scope" parameter
               list.scopeSpecified =true; 
               //Set scope
               list.scope = scope;
               foreach (unitType utype in infrastruct.list(list).unit)
               {
                   list_result +="Name: " + utype.name+"\n";
                   list_result +="ID: " + utype.id+"\n";
                   list_result +="Eid: " + utype.eid +"\n";
                   if (utype.eid != null)
                   {
                       foreach (string eid in utype.eid)
                       {
                           list_result +=eid+"\n";
                       }
                   }
                   else
                       list_result +="No eids!\n"; 
                   list_result +="Units:\n";
                   if (utype.unit != null)
                   {
                       foreach (unitType utype1 in utype.unit )
                       {
                           list_result +="Name: " + utype1.name+"\n";
                           list_result +="Eids: \n";
                           if (utype1.eid != null)
                           {
                               foreach (string eid in utype1.eid)
                               {
                                   list_result +=eid+"\n";
                               }
                           }
                           else
                              list_result +="No eids!\n"; 
                           
                       }
                   }
                   else list_result +="No units\n";

               }
               return list_result; 
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }
            
       ///<summary>
       ///  Method to add environment to unit
       /// <param name="ve_eid">VE to add</param>
       /// <param name="unit_name">Unit name</param>
       /// <returns>
       /// <para>Returns "OK" if adding was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       ///  <para>Example: addEnv("ac57a9c3-573b-481a-b398-d2fb0467cf4b","ou=SoftDev,o=VZLOrganization,dc=vzl") </para>
       /// </example>
       ///</summary>
       public string addEnv(string ve_eid,string unit_name,string base_name)
       {
           try
           {
               if (infrastruct == null) infrastruct = (infrastructuremBinding)binder.InitBinding(typeof(infrastructuremBinding));
              
               add_env env=new add_env (); 
               //Set unit name
               env.name = unit_name+","+base_name;       
               //Set number of VE to add
               env.eid =new string [1];    
               //Set VE eid for each VE
               env.eid[0] = ve_eid;
               infrastruct.add_env(env);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       ///  Method to move environment from one unit to another
       /// <param name="ve_eid">VE EID</param>
       /// <param name="source">Source unit</param>
       /// <param name="destination">Destination unit</param>
       /// <returns>
       /// <para>Returns "OK" if moving was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       ///  <para>Example: moveEnv("ac57a9c3-573b-481a-b398-d2fb0467cf4b","ou=SoftDev,o=VZLOrganization,dc=vzl","ou=QA,o=VZLOrganization,dc=vzl") </para>
       /// </example>
       ///</summary>
       public string moveEnv(string ve_eid,string source, string destination,string base_name)
       {
           try
           {
               if (infrastruct == null) infrastruct = (infrastructuremBinding)binder.InitBinding(typeof(infrastructuremBinding));
             
               move_env env = new move_env();
               //Set source
               env.src = source+","+base_name ; 
               //Set VE
               env.eid = ve_eid; 
               //Set destination
               env.dst = destination + "," + base_name;
               infrastruct.move_env(env);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

    }
}
