using System;
using System.Collections.Generic;
using System.Text;            
using SOAPClasses.VZA ;

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with perfmon.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="getCountersInfo" /> to get counters and info.</item>
    ///     </list>
    /// </remarks>
    public class PerfMon
    {
       perf_monBinding perf_mon;
       Binder binder; 

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public PerfMon(Binder _binder)
        {
            binder = _binder;
                       
        }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public PerfMon(string url, string session)
        {
            binder = new Binder(url, session);
        }

       
       ///<summary>
       /// Method to get counters and info
       /// <param name="class_name">Counter's class name
       /// <para>counters_vz_cpu</para>
       /// <para>counters_vz_net</para>
       /// <para>counters_vz_loadavg</para>
       /// <para>counters_vz_process</para>
       /// <para>counters_vz_slm</para>
       /// <para>counters_vz_system</para>
       /// <para>counters_vz_memory</para>
       /// <para>counters_vz_hw_net</para>
       /// <para>counters_vz_quota</para>
       /// <para>counters_vz_ubc</para>
       /// </param>
       /// <param name="counter_name">Counter name (for counters_vz_cpu):
        /// <para>counter_cpu_system</para>
        /// <para>counter_cpu_user</para>
        /// <para>counter_cpu_idle</para>
        /// <para>counter_cpu_nice</para>
        /// <para>counter_cpu_starvation</para>
        /// <para>counter_cpu_system_states</para>
        /// <para>counter_cpu_user_states</para>
        /// <para>counter_cpu_idle_states</para>
        /// <para>counter_cpu_nice_states</para>
        /// <para>etc</para>
        /// </param>
       /// <returns>
       /// <para>Returns information about counters.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getCountersInfo("counters_vz_cpu","counter_cpu_system")</para>
       /// </example>
       ///</summary>
       public string getCountersInfo(string class_name, string counter_name)
       {
           string counters_info = "";
           try
           {
               if (perf_mon == null) perf_mon = (perf_monBinding)binder.InitBinding(typeof(perf_monBinding));

               get4 get = new get4();
               get.@class = new classType1[1];
               get.@class[0] = new classType1();
               get.eid_list = new string[1];
               get.@class[0].name = class_name;
               get.@class[0].instance = new classTypeInstance[1];
               get.@class[0].instance[0] = new classTypeInstance();
               get.@class[0].instance[0].counter = new string[1];
               get.@class[0].instance[0].counter[0] = counter_name;

               getData(perf_mon.get(get), out counters_info);
               return counters_info;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
           catch (Exception e)
           {
               return "Exception: " + e.Message;
           }
       }

       ///<summary>
       /// Method to get counters and info
       /// <param name="class_name">Counter's class name
       /// </param>
       /// <param name="counter_name">Counter name (for counters_vz_cpu):
      /// </param>
       /// <returns>
       /// <para>Returns information about counters.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getCounterAvgValue("counters_vz_cpu","counter_cpu_system")</para>
       /// </example>
       ///</summary>
       public string getCounterAvgValue(string ve_eid,string class_name, string counter_name)
       {
          try
           {
               if (perf_mon == null) perf_mon = (perf_monBinding)binder.InitBinding(typeof(perf_monBinding));

               get4 get = new get4();
               get.@class = new classType1[1];
               get.@class[0] = new classType1();
               get.eid_list = new string[1];
               get.eid_list[0] = ve_eid;
               get.@class[0].name = class_name;
               get.@class[0].instance = new classTypeInstance[1];
               get.@class[0].instance[0] = new classTypeInstance();
               get.@class[0].instance[0].counter = new string[1];
               get.@class[0].instance[0].counter[0] = counter_name;

               perf_dataType[] data = perf_mon.get(get);

               if (data.Length != 0)
                   return data[0].@class[0].instance[0].counter[0].value.avg;
               else
                   return "Exception: no available data!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
           catch (Exception e)
           {
               return "Exception: " + e.Message;
           }
       }

       ///<summary>
       /// Method to write data to string
       /// <param name="counters_dat">Data</param>
       /// <param name="counters_info">String to which write data</param>
       /// <example>
       /// <para>Example: getData(counters_dat,out counters_info)</para>
       /// </example>
       ///</summary>
       public void getData(perf_dataType[] counters_dat,out string counters_info )
       {
           counters_info = "";
           if (counters_dat.Length != 0)
           {
               foreach (perf_dataType counter_dat in counters_dat)
               {
                   if (counter_dat.@class != null)
                   {
                       foreach (perf_dataTypeClass dat in counter_dat.@class)
                       {
                           counters_info += "\n Class name: " + dat.name + "\n" +
                           "Instances:\n";
                           if (dat.instance != null)
                           {
                               foreach (perf_dataTypeClassInstance instance in dat.instance)
                               {
                                   counters_info += " DataClassInstance: " + instance.name + "\n";
                                   if (instance.counter != null)
                                   {
                                       foreach (perf_dataTypeClassInstanceCounter counter in instance.counter)
                                       {
                                           counters_info += "     \nName:" + counter.name + "\n" +
                                           "        Value avg: " + counter.value.avg + "\n" +
                                           "        Value cur: " + counter.value.cur + "\n" +
                                           "        Value max: " + counter.value.max + "\n" +
                                           "        Value min: " + counter.value.min;

                                       }
                                   }
                                   else
                                       counters_info += " No counters!" + "\n";
                               }
                           }
                           else
                               counters_info += "No instances!" + "\n";
                       }
                   }
                   else
                       counters_info += "No classes!" + "\n";
                   counters_info += "Intervals:\n" +
                   "Start time: " + counter_dat.interval.start_time + "\n" +
                   "End time: " + counter_dat.interval.end_time + "\n" +
                   "EID: " + counter_dat.eid + "\n";
               }
           }
           else
           {
               counters_info += "No info about such counters!";
               throw new Exception("Test Exception: No info about such counters! ");
           }
       }
    }
}
