using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 

namespace SOAPClasses
{
   /// <summary>
   /// This class have examples how to work with pkgm.
   /// </summary>
   /// <remarks>
   ///		<para>The current class includes:</para>
   ///		<list type="bullet">
   ///			<item><see cref="installPackage" /> to install package.</item>
   ///          <item><see cref="updatePackage" /> to update package.</item>
   ///          <item><see cref="removePackage" /> to remove package.</item>
   ///          <item><see cref="getPackageInfo" /> to get information about package.</item>
   ///          <item><see cref="getPackageListForVE" /> to get list of packages on VE.</item>
   ///          <item><see cref="getPackageList" /> to get list of packages.</item>
   ///          <item><see cref="getOSTemplates" /> to get OS templates.</item>
   ///     </list>
   /// </remarks>
   public class Pkgm
    {
       vzapackagemBinding pkgm;
       Binder binder;

       ///<summary>
       ///  Constructor
       /// <param name="Binder">Binder</param>
       ///</summary>
       public Pkgm(Binder _binder)
       {
            binder = _binder;                       
       }

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Pkgm(string url, string session)
       {
           binder = new Binder(url, session);
       }

      ///<summary>
      /// Method to install package to VE
      /// <param name="ve_eid">VE EID</param>
      /// <param name="pkg_name">Package name</param>
      /// <param name="pkg_version">Package version</param>
      /// <example>
      /// <para>Example: installPackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64", "20060313")</para>
      /// </example>
      ///</summary>
      public string installPackage(string ve_eid,string pkg_name,string pkg_version)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               install3 install = new install3();
                
               package_std_vztemplateType package = new package_std_vztemplateType();
               //Set package name
               package.name = pkg_name;
               //Set package version
               package.version = pkg_version;

               //Set VE EID
               install.eid = ve_eid;
               install.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               install.installation_package[0] = new pkg_setup_cmdTypeInstallation_package();
               install.installation_package[0].Item = package;
               pkgm.install (install);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       ///<summary>
       /// Method to install EZ package to VE
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <example>
       /// <para>Example: installPackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64")</para>
       /// </example>
       ///</summary>
       public string installEZPackage(string ve_eid, string pkg_name)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               install3 install = new install3();

               //package_std_vztemplateType package = new package_std_vztemplateType();
               package_vztemplateType package = new package_vztemplateType();
               ////Set package name
               package.name = pkg_name;
              

               //Set VE EID
               install.eid = ve_eid;
               install.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               install.installation_package[0] = new pkg_setup_cmdTypeInstallation_package();
               install.installation_package[0].Item = package;
               pkgm.install(install);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       ///<summary>
       /// Method to install EZ package to VE
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <example>
       /// <para>Example: installPackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64")</para>
       /// </example>
       ///</summary>
       public string installEZPackageOnHWNode(string pkg_name)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               install3 install = new install3();

               package_linuxType package = new package_linuxType();
               package.path = System.Text.ASCIIEncoding.ASCII.GetBytes(pkg_name); 
              
               //Set VE EID
               install.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               install.installation_package[0] = new pkg_setup_cmdTypeInstallation_package();
               install.installation_package[0].Item = package;
               install.options = new pkg_setup_cmdTypeOptions();
               install.options.force = new object(); 
               pkgm.install(install);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to update package to VE
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <param name="pkg_version">Package version</param>
       /// <example>
       /// <para>Example: updatePackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64", "20060513")</para>
       /// </example>
       ///</summary>
       public string updatePackage(string ve_eid, string pkg_name, string pkg_version)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               update2 update = new update2();

               pkg_setup_cmdTypeInstallation_package install_package = new pkg_setup_cmdTypeInstallation_package();
               
               package_std_vztemplateType package = new package_std_vztemplateType();
               //Set package name
               package.name = pkg_name;
               //Set package version
               package.version = pkg_version; 
              
               install_package.Item = package;
               update.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               update.installation_package[0] = install_package;
               //Set VE EID
               update.eid = ve_eid;
               pkgm.update(update);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to update package to HWNode
       /// <param name="ve_eid">VE EID</param>
       /// <param name="path">Path to the package</param>
       /// <example>
       /// <para>Example: updatePackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","/tmp/tomcat5030-as4-tmpl-3.0.0-2.swsoft.i386.rpm")</para>
       /// </example>
       ///</summary>
       public string updatePackageWithPath(string ve_eid,string path)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               update2 update = new update2();

               pkg_setup_cmdTypeInstallation_package install_package = new pkg_setup_cmdTypeInstallation_package();

               //Set path to the package
               install_package.Item = System.Text.ASCIIEncoding.ASCII.GetBytes(path);
               update.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               update.installation_package[0] = install_package;

               //Set VE EID
               update.eid = ve_eid;
               pkgm.update(update);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to update package to HWNode
       /// <param name="ve_eid">VE EID</param>
       /// <param name="path">Path to the package</param>
       /// <example>
       /// <para>Example: installPackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","/tmp/tomcat5030-as4-tmpl-3.0.0-2.swsoft.i386.rpm")</para>
       /// </example>
       ///</summary>
       public string installPackageWithPath(string ve_eid, string path)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               install3 install = new install3();

               pkg_setup_cmdTypeInstallation_package install_package = new pkg_setup_cmdTypeInstallation_package();
               
               //Set path to the package
               install_package.Item = System.Text.ASCIIEncoding.ASCII.GetBytes(path);
               install.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               install.installation_package[0] = install_package;

               //Set VE EID
               install.eid = ve_eid;
               pkgm.install(install)   ;

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to install package to HWNode
       /// <param name="path">Path to the package</param>
       /// <example>
       /// <para>Example: installPackage("/tmp/tomcat5030-as4-tmpl-3.0.0-2.swsoft.i386.rpm")</para>
       /// </example>
       ///</summary>
       public string installPackageOnHWNode(string path)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               install3 install = new install3();

               pkg_setup_cmdTypeInstallation_package install_package = new pkg_setup_cmdTypeInstallation_package();

               //Set path to the package
               install_package.Item = System.Text.ASCIIEncoding.ASCII.GetBytes(path);
               install.installation_package = new pkg_setup_cmdTypeInstallation_package[1];
               install.installation_package[0] = install_package;
               
               pkgm.install (install);

               return "OK"  ;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to remove package from HWNode
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <param name="pkg_version">Package version</param>
       /// <example>
       /// <para>Example: removePackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64", "20060513")</para>
       /// </example>
       ///</summary>
       public string removePackage(string ve_eid, string pkg_name, string version)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               remove2 remove = new remove2();

               remove.packages= new packagesType ();
               remove.packages.Items = new package_std_vztemplateType[1];
               remove.packages.Items[0] = new package_std_vztemplateType();
               //Set package name
               remove.packages.Items[0].name = pkg_name;
               //Set package version
               remove.packages.Items[0].version = version;

               //Set VE EID
               remove.eid = ve_eid;

               pkgm.remove(remove);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }


       ///<summary>
       /// Method to remove EZ package from VE 
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <example>
       /// <para>Example: removePackage("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64")</para>
       /// </example>
       ///</summary>
       public string removeEZPackage(string ve_eid, string pkg_name)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               remove2 remove = new remove2();

               remove.packages = new packagesType();
               remove.packages.Items = new package_std_vztemplateType[1];
               remove.packages.Items[0] = new package_std_vztemplateType();
               //Set package name
               remove.packages.Items[0].name = pkg_name;
             
               //Set VE EID
               remove.eid = ve_eid;

               pkgm.remove(remove);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get information about package
       /// <param name="ve_eid">VE EID</param>
       /// <param name="pkg_name">Package name</param>
       /// <param name="pkg_version">Package version</param>
       /// <example>
       /// <para>Example: getPackageInfo("ac57a9c3-573b-481a-b398-d2fb0467cf4b","mailman-as4-x86_64", "20060513")</para>
       /// </example>
       ///</summary>
       public string getPackageInfo(string ve_eid, string pkg_name)
       {
           try
           {
               string info_result = "";
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               get_info1 get_info=new get_info1 ();
               //Set VE EID
               get_info.eid = ve_eid;
               get_info.packages = new packagesType ();
               get_info.packages.Items = new package_std_vztemplateType[1];
               get_info.packages.Items[0] = new package_std_vztemplateType();
               //Set package name
               get_info.packages.Items[0].name = pkg_name;
              
               packageType[] packages_info = pkgm.get_info(get_info).packages.Items;
               if (packages_info != null)
               {
                   foreach (packageType pack in packages_info)
                   {
                       info_result+="\n\nName: " + pack.name+
                       "\nDescription: " + pack.description+
                      "\nVersion: " + pack.version;

                   }
               }
               else
                  return "Package not found!";
              return info_result;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get list of packages from VE
       /// <param name="ve_eid">VE EID</param>
       /// <example>
       /// <para>Example: getPackageListForVEac57a9c3-573b-481a-b398-d2fb0467cf4b")</para>
       /// </example>
       ///</summary>
       public string getPackageListForVE(string ve_eid)
       {
           try
           {
               string list_result = "";
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               list5 pkg_list=new list5(); 
               //Set VE EID
               pkg_list.eid =ve_eid;

               packagesType pkgs = pkgm.list(pkg_list).packages;
               if (pkgs.Items != null || pkgs.Items.Length != 0)
               {
                  
                   foreach (packageType package in pkgs.Items)
                   {
                       if (package is package_std_vztemplateType)
                           list_result +="\nType: package_std_vztemplate";
                       if (package is package_rpmType )
                           list_result +="\nType: package_rpm";
                       if (package is package_debType)
                          list_result +="\nType: package_deb";

                      list_result += "\nName: " + package.name +
                      "\nDescription: " + package.description +
                      "\nVersion: " + package.version;
                   }
               }
               else
                   return "No packages!";
               return list_result;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get list of packages
       /// <example>
       /// <para>Example: getPackageList()</para>
       /// </example>
       ///</summary>
       public string getAllPackageList(string param)
       {
           try
           {
               string list_result = "";
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               list5 pkg_list = new list5();
             
               packagesType pkgs = pkgm.list(pkg_list).packages;
               if (pkgs.Items != null || pkgs.Items.Length != 0)
               {

                   foreach (packageType package in pkgs.Items)
                   {
                       if (package is package_std_vztemplateType)
                           list_result += "\nType: package_std_vztemplate";
                       if (package is package_rpmType)
                           list_result += "\nType: package_rpm";
                       if (package is package_debType)
                           list_result += "\nType: package_deb";

                       list_result += "\nName: " + package.name +
                       "\nDescription: " + package.description +
                       "\nVersion: " + package.version;
                   }
               }
               else
                   return "No packages!";
               return list_result;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

      


       ///<summary>
       /// Method to get list of packages for OS
       /// <param name="OS">OS</param>
       /// <example>
       /// <para>Example: getPackageList("redhat-as3")</para>
       /// </example>
       ///</summary>
       public string getPackageList(string OS)
       {
           try
           {
               if (pkgm == null) pkgm = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               string packages = "";
               list5 pkg_list = new list5();
               packagesType pkgs = pkgm.list(pkg_list).packages;
               if (pkgs.Items != null && pkgs.Items.Length != 0)
               {
                   foreach (packageType package in pkgs.Items)
                   {
                       //Check OS name
                       if (package.os.platform == OS)
                           packages+=package.name + "," + package.version+"\n";
                   }
               }
               else
                   return "";
               return packages;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get list of OS templates
       /// <example>
       /// <para>Example: getOSTemplates()</para>
       /// </example>
       ///</summary>
       public List<string> getOSTemplates()
       {
           try
           {
               List<string > os_templates=new List<string> ();
               vzapackagemBinding vzapkg = (vzapackagemBinding)binder.InitBinding(typeof(vzapackagemBinding));
               list5 pkg_list=new list5 ();

               packagesType pkgs =vzapkg.list(pkg_list).packages;  
                
               foreach (packageType package in pkgs.Items)
               {
                   if (package is package_std_vztemplateType)
                   {
                       //Check if package is os template
                       if (((package_std_vztemplateType)package).os_template)
                           os_templates.Add(package.name);
                   }
                  
               }

               return os_templates;
           }
           catch
           {
               return null;
           }
       }
    }
}
