using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with processm.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="execute" /> to execute a program.</item>
    ///     </list>
    /// </remarks>
   public class Processm
    {
       processmBinding process;
       Binder binder;

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       /// <param name="commands">Commands to execute</param>
       /// <param name="dir">Working directory</param>
       /// <returns>
       /// <para>Returns "OK" if execution was successful or "Exception" if not.</para>                            
       ///</summary>
       public Processm(Binder _binder)
       {
           binder = _binder;
       }

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Processm(string url, string session)
       {
           binder = new Binder(url, session);
       }     

      
       ///<summary>
       /// Method to execute a program
       /// </returns>
       /// <example>
       /// <para>Example: execute(commands,"/")</para>
       /// </example>
       ///</summary>
       public string execute(string commander,string dir)
       {
           string exec_result = "";
           try
           {
               if (process == null) process = (processmBinding)binder.InitBinding(typeof(processmBinding));
              
               execute exec = new execute();
               string[] commands = commander.Split(new char[] { ' ' }); 
               byte[][] command = new byte[commands.Length][]; 

               //Set commands to execute
               for (int i = 0; i < commands.Length;i++ )
                   command[i] = System.Text.ASCIIEncoding.ASCII.GetBytes(commands[i]);
               exec.argv = command;
               exec.cred = new executeCred();
  
               //Set working directory
               exec.cred.chdir = System.Text.ASCIIEncoding.ASCII.GetBytes(dir);
               exec.stdio = new executeStdio();
               exec.stdio.error = new object();  // to get errors
               exec.stdio.output = new object(); // to get outputs
               exec result=process.execute(exec).exec ;
               exec_result +="\nError: " + System.Text.ASCIIEncoding.ASCII.GetString(result.error)+
               "\nOutput: " + System.Text.ASCIIEncoding.ASCII.GetString(result.output )+
               "\nStatus: " + result.status.ToString ();
               return exec_result;

           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       public void addExecuteEventHandler(executeCompletedEventHandler e)
       {
           if (process == null) 
               process = (processmBinding)binder.InitBinding(typeof(processmBinding));
           process.executeCompleted += e;
       }

       public void executeAsync(string commander, string dir, object tag)
       {
           if (process == null)
               process = (processmBinding)binder.InitBinding(typeof(processmBinding));

           execute exec = new execute();
           string[] commands = commander.Split(new char[] { ' ' });
           byte[][] command = new byte[commands.Length][];

           //Set commands to execute
           for (int i = 0; i < commands.Length; i++)
               command[i] = System.Text.ASCIIEncoding.ASCII.GetBytes(commands[i]);
           exec.argv = command;
           exec.cred = new executeCred();

           //Set working directory
           exec.cred.chdir = System.Text.ASCIIEncoding.ASCII.GetBytes(dir);
           exec.stdio = new executeStdio();
           exec.stdio.error = new object();  // to get errors
           exec.stdio.output = new object(); // to get outputs
           process.executeAsync(exec, tag);
       }

       public string testForBug100785()
       {
           try
           {

               // Start sleep process async
               Random rand = new Random(123);

               int sleepTime = rand.Next(1000, 5000);
               string cmdLine = "sleep " + sleepTime;
               Envm env = new Envm(binder);
               eidList eids = env.getEIDList(6);
               if (eids.Count == 0)
                   throw new System.Exception("no running CT");
               
               string eid = eids[0];
               int veid = env.getVEID(eid);
               string cmdLineFull = "vzctl exec " + veid + " " + cmdLine;
               executeAsync(cmdLineFull, "", new object());
               // Wait until process started
               System.Threading.Thread.Sleep(15000);
               // get started process PID
               ProcInfo pi = new ProcInfo(binder);
               int pid = pi.getPID(eid, cmdLine);
               // kill by pid
               vzaprocessmBinding vzproc = (vzaprocessmBinding) binder.InitBinding(typeof(vzaprocessmBinding));
               kill1 karg = new kill1();
               karg.eid = eid;
               karg.pid = new int[1] { pid };
               karg.signal = 9;
               vzproc.kill(karg);
               return "Ok!";

           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }

       }

    }
}
