using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA; 

namespace SOAPClasses
{
    /// <summary>
    /// This class have examples how to work with relocator.
    /// </summary>
    /// <remarks>
    ///		<para>The current class includes:</para>
    ///		<list type="bullet">
    ///			<item><see cref="Migrate" /> to migrate VE from one node to another.</item>
    ///     </list>
    /// </remarks>
    public class Relocator
    {
       relocatorBinding relocator;
       Binder binder;
       
       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public Relocator(Binder _binder)
       {
           binder = _binder;
           
       }

        ///<summary>
        ///  Constructor
        /// <param name="sURL">URL</param>
        /// <param name="sess">session</param>
        ///</summary>
        public Relocator(string url, string session)
        {
            binder = new Binder(url, session);
          
        }

      
       ///<summary>
       /// Method to migrate VE from one node to another
       /// <param name="ve_eid">VE EID</param>
       /// <param name="ip_address">Destination IP address</param>
       /// <param name="port">Port</param>
       /// <param name="protocol">Protocol</param>
       /// <param name="username">User name</param>
       /// <param name="realm">Realm</param>
       /// <param name="domain">Domain</param>
       /// <param name="password">Password</param>
       /// <returns>
       /// <para>Returns "OK" migration was successfull or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: Migrate("ac57a9c3-573b-481a-b398-d2fb0467cf4b", "10.16.0.88", 4433, "TCP", "Administrateur", "00000000-0000-0000-0000-000000000000", "", "1q2w3e")</para>
       /// </example>
       ///</summary>
       public string migrate(string ve_eid,string ip_address,uint port,string protocol,string username,string realm,string domain,string password)
       {
           try
           {
               if (relocator == null) relocator = (relocatorBinding)binder.InitBinding(typeof(relocatorBinding),"vzarelocator");
               migrate_v2v mig_v2v = new migrate_v2v();
               //Set VE EID
               mig_v2v.eid_list = new string[1] ;
               mig_v2v.eid_list[0]=ve_eid;
               mig_v2v.dst = new connection_infoType();
               //Set IP address
               connection_infoType  migrat_param=(connection_infoType)mig_v2v.dst;

               migrat_param.address = ip_address;
               //Allow to specify port
               migrat_param.portSpecified = true;
               //Set port
               migrat_param.port = port;
               //Set protocol
               migrat_param.protocol = protocol;
               //Set login parameters
               migrat_param.login = new auth_nameType();
               migrat_param.login.name = System.Text.ASCIIEncoding.ASCII.GetBytes(username);
               migrat_param.login.realm = realm;
               migrat_param.login.domain = System.Text.ASCIIEncoding.ASCII.GetBytes(domain);
               //Set password
               migrat_param.password = System.Text.ASCIIEncoding.ASCII.GetBytes(password);

               // Infinite timeout
               relocator.Timeout = -1;
               relocator.migrate_v2v(mig_v2v);

               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to migrate VE from one node to another
       /// <param name="ve_eid">VE EID</param>
       /// <param name="count">Count</param>
       /// <returns>
       /// <para>Returns "OK" if clonning was successfull or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: Clone("ac57a9c3-573b-481a-b398-d2fb0467cf4b", 2)</para>
       /// </example>
       ///</summary>
       public string clone(string ve_eid,int count)
       {
           try
           {
               if (relocator == null) relocator = (relocatorBinding)binder.InitBinding(typeof(relocatorBinding),"vzarelocator");
               clone cloner = new clone();
               //Set VE EID
               cloner.eid = ve_eid;
               //Set count of clones
               cloner.count = count;               
               // Infinite timeout
               relocator.Timeout = -1;
               relocator.clone(cloner);                  
              
               return "OK";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText ;
           }
       }
    }
}
