using System;
using System.Collections.Generic;
using System.Text;
using SOAPClasses.VZA;  


namespace SOAPClasses
{
   /// <summary>
   /// This class have examples how to work with reslog.
   /// </summary>
   /// <remarks>
   ///		<para>The current class includes:</para>
   ///		<list type="bullet">
   ///			<item><see cref="setLog" /> to set logging.</item>
   ///			<item><see cref="getLog" /> to get result log.</item>
   ///     </list>
   /// </remarks>
   public class ResLog
    {
       res_logBinding reslog;
       Binder binder;

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public ResLog(Binder _binder)
       {
           binder = _binder;
       }

       ///<summary>
       ///  Constructor
       /// <param name="sURL">URL</param>
       /// <param name="sess">session</param>
       ///</summary>
       public ResLog(string url, string session)
       {
           binder = new Binder(url, session);
       }


       ///<summary>
       /// Method to set logging
       /// <param name="class_name">Counter's class name 
       /// <para>counters_vz_cpu</para>
       /// <para>counters_vz_net</para>
       /// <para>counters_vz_loadavg</para>
       /// <para>counters_vz_process</para>
       /// <para>counters_vz_slm</para>
       /// <para>counters_vz_system</para>
       /// <para>counters_vz_memory</para>
       /// <para>counters_vz_hw_net</para>
       /// <para>counters_vz_quota</para>
       /// <para>counters_vz_ubc</para>
       /// </param>
       /// <param name="log_period">Logging period (ms) </param>
       /// <returns>
       /// <para>Returns "OK" if setting was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: setLog("counters_vz_cpu",800)</para>
       /// </example>
       ///</summary>
       public string setLog(string class_name,int log_period)
       {
           try
           {
               if (reslog == null) reslog = (res_logBinding)binder.InitBinding(typeof(res_logBinding));
              
               logType logtype = new logType();
               //Set logging period
               logtype.period = log_period;
               logTypeClass[] classes = new logTypeClass[1];
               classes[0] = new logTypeClass();
               //Set counter's class  
               classes[0].name = class_name;
               logtype.@class = classes;               
               reslog.set_log(logtype);

               return "OK!";
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get list of the top resource consuming Environments.
       /// <param name="count">Count </param>
       /// <param name="class_name">Counter's name 
       /// <para>counters_vz_cpu</para>
       /// <para>counters_vz_net</para>
       /// <para>counters_vz_loadavg</para>
       /// <para>counters_vz_process</para>
       /// <para>counters_vz_slm</para>
       /// <para>counters_vz_system</para>
       /// <para>counters_vz_memory</para>
       /// <para>counters_vz_hw_net</para>
       /// <para>counters_vz_quota</para>
       /// <para>counters_vz_ubc</para>
       /// </param>
       /// <returns>
       /// <para>Returns "OK" if getting was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getTop("ac57a9c3-573b-481a-b398-d2fb0467cf4b",30,"counters_vz_cpu")</para>
       /// </example>
       ///</summary>
       public string getTop(string ve_eid, int count, string counter_name)
       {
           try
           {
              if (reslog == null) reslog = (res_logBinding)binder.InitBinding(typeof(res_logBinding));

               get_top gettop = new get_top();
               //Set counter class
               gettop.parameter = counter_name;
               //Set VE eid
               gettop.eid_list = new string[1];
               gettop.eid_list[0] = ve_eid;
               //Set count
               gettop.countSpecified = true;
               gettop.count = count; 
 
               return  reslog.get_top(gettop).top.total  ;
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText ;
           }
       }

       ///<summary>
       /// Method to get list of the top resource consuming Environments.
       /// <param name="ve_eid">VE eid</param>
       /// <param name="start_time">Start time</param>
       /// <param name="end_time">End time</param>
       /// <param name="class_name">Counter's class name 
       /// <para>counters_vz_cpu</para>
       /// <para>counters_vz_net</para>
       /// <para>counters_vz_loadavg</para>
       /// <para>counters_vz_process</para>
       /// <para>counters_vz_slm</para>
       /// <para>counters_vz_system</para>
       /// <para>counters_vz_memory</para>
       /// <para>counters_vz_hw_net</para>
       /// <para>counters_vz_quota</para>
       /// <para>counters_vz_ubc</para>
       /// </param>
       /// <param name="log_period">Logging period (ms) </param>
       /// <returns>
       /// <para>Returns "OK" if getting was successful or "Exception" if not.</para>
       /// </returns>
       /// <example>
       /// <para>Example: getTop("ac57a9c3-573b-481a-b398-d2fb0467cf4b","02.05.2007 08:00:03 AM","07.05.2007 08:00:03 PM ","counters_vz_cpu")</para>
       /// </example>
       ///</summary>
       public string getTopForTime(string ve_eid, string start_time, string end_time, string class_name)
       {
           try
           {
              if (reslog == null) reslog = (res_logBinding)binder.InitBinding(typeof(res_logBinding));

               get_top gettop = new get_top();
               //Set counter class
               gettop.parameter = class_name;
               //Set VE eid
               gettop.eid_list = new string[1];
               gettop.eid_list[0] = ve_eid;
               //Set start time
               gettop.start_time = Convert.ToDateTime(start_time).ToString("s");
               //Set finish time
               gettop.end_time = Convert.ToDateTime(end_time).ToString("s");
               
               return reslog.get_top(gettop).top.total;
               
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }
       }

       ///<summary>
       /// Method to get log (from start_time to end_time for "counters_cpu")
       /// <param name="ve_eid">VE eid</param>
       /// <param name="start_time">Start time</param>
       /// <param name="end_time">End time</param>
       /// <param name="class_name">Counter's class name 
       /// <para>counters_vz_cpu</para>
       /// <para>counters_vz_net</para>
       /// <para>counters_vz_loadavg</para>
       /// <para>counters_vz_process</para>
       /// <para>counters_vz_slm</para>
       /// <para>counters_vz_system</para>
       /// <para>counters_vz_memory</para>
       /// <para>counters_vz_hw_net</para>
       /// <para>counters_vz_quota</para>
       /// <para>counters_vz_ubc</para>
       /// </param>
       /// <returns>
       /// <para>Returns resource log .</para>
       /// </returns>
       /// <example>
       /// <para>Example: getLog("ac57a9c3-573b-481a-b398-d2fb0467cf4b","02.05.2007 08:00:03 AM","07.05.2007 08:00:03 PM ","counters_vz_cpu")</para>
       /// </example>
       ///</summary>
       public string getLog(string ve_eid,string start_time, string end_time, string class_name)
       {
           string log_info="";
           try
           {
               if (reslog == null) reslog = (res_logBinding)binder.InitBinding(typeof(res_logBinding));
               
               get_log logtype = new get_log();
               Envm envm = new Envm(binder);
               logtype.eid =ve_eid;
               //Set start time
               logtype.start_time = Convert.ToDateTime(start_time).ToString("s"); // convert to ISO-8601 time format 
               classType[] classes=new classType [1];
               classes[0]=new classType ();
               //Set counter class
               classes[0].name = class_name;
               logtype.@class = classes;
               //Set end time
               logtype.end_time = Convert.ToDateTime(end_time).ToString("s");  // convert to ISO-8601 time format 
               //Set infinite timeout
               reslog.Timeout = -1;
               //Get resource log
               PerfMon perfmon = new PerfMon(binder);
               perfmon.getData(reslog.get_log(logtype), out log_info);

               return log_info;             
           }
           catch (System.Web.Services.Protocols.SoapException e)
           {
               return "Exception: " + e.Detail.InnerText;
           }            
       }
    }
}
