using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using SOAPClasses;
using System.Reflection;
using System.Threading;

namespace SOAPconsole
{
    class CreateMethodContainer
    {
        public static object createMethodContainer(Type operatorType, object[] parameters)
        {
            object construct = null;

            if (operatorType == null)
                return construct;
            //   Get all constructors
            ConstructorInfo[] ci = operatorType.GetConstructors();
            if (ci.Length != 0)
            {
                foreach (ConstructorInfo constructor in operatorType.GetConstructors())
                {
                    if (constructor.GetParameters().Length == parameters.Length)
                    {
                        try
                        {
                            construct = constructor.Invoke(parameters);
                        }
                        catch
                        {
                            construct = null;
                        }
                    }
                    else if (constructor.GetParameters().Length == 0)
                    {
                        construct = constructor.Invoke(null);
                    }
                }
            }
            return construct;
        }

        public static object createMethodContainer(Type type)
        {
            object res = null;

            if (type == null)
                return res;

            ConstructorInfo[] ci = type.GetConstructors();
            if (ci.Length != 0)
            {
                ParameterInfo[] pi = ci[0].GetParameters();
                try
                {
                    res = ci[0].Invoke(null);
                }
                catch
                {
                    res = null;
                }
            }
            return res;
        }
    }

    class CreateVE : CreateMethodContainer
    {
        public bool error = false;
        public string ve_eid = "";
        public string errorMessage = "";

        private object url;
        private List<object> argsConnection;
        private List<object> argsCreateVE;
        private ManualResetEvent doneEvent;
        private string password;

        public CreateVE(object url_, List<object> argsConnection_, List<object> argsCreateVE_, string password_, ManualResetEvent doneEvent_)
        {
            url = url_;
            argsConnection = argsConnection_;
            argsCreateVE = argsCreateVE_;
            doneEvent = doneEvent_;
            errorMessage = "No errors occurred";
            password = password_;
        }

        public void createVEProc(Object threadContext)
        {
            int threadIndex = (int)threadContext;

            try
            {
                Console.WriteLine("Thread {0}: started...", threadIndex);

                string threadSession;
                Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
                
                Type connectType = assembly.GetType("SOAPClasses.Connect");
                object connectConnector = createMethodContainer(connectType);

                // create per-thread session
                try
                {
                    threadSession = connectType.GetMethod("Login").Invoke(connectConnector, argsConnection.ToArray()).ToString();
                    Console.WriteLine("Thread {0}: Logged in", threadIndex, ve_eid);
                }
                catch (ApplicationException e)
                {
                    if(e.InnerException != null)
                        throw new ApplicationException("Failed to create session. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to create session. " + e.Message);
                }

                Type envmType = assembly.GetType("SOAPClasses.Envm");
                object envmConnector = createMethodContainer(envmType, new object[] {url, threadSession});

                // create VE            
                try
                {
                    ve_eid = envmType.GetMethod("createVE").Invoke(envmConnector, argsCreateVE.ToArray()).ToString();

                    Console.WriteLine("Thread {0}: Successfully created CT {1}", threadIndex, ve_eid);
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to execute method Envm::createVE. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method Envm::createVE. " + e.Message);
                }

                List<object> argsVEEID = new List<object>();
                argsVEEID.Add(ve_eid);

                string status = "";

                // getVEState
                try
                {
                    status = envmType.GetMethod("getVEState").Invoke(envmConnector, argsVEEID.ToArray()).ToString();
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to execute method Envm::getVEState. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method Envm::getVEState. " + e.Message);
                }

                // start CT if it is not running
                if (status == "6")
                {
                    Console.WriteLine("Thread {0}: CT {1} is running", threadIndex, ve_eid);
                }
                else
                {
                    Console.WriteLine("Thread {0}: CT {1} is stopped, starting", threadIndex, ve_eid);

                    // startVE
                    try
                    {
                        envmType.GetMethod("startVE").Invoke(envmConnector, argsVEEID.ToArray()).ToString();
                        Console.WriteLine("Thread {0}: CT {1} is running", threadIndex, ve_eid);
                    }
                    catch (ApplicationException e)
                    {
                        if (e.InnerException != null)
                            throw new ApplicationException("Failed to execute method Envm::startVE. " + e.InnerException.Message);
                        else
                            throw new ApplicationException("Failed to execute method Envm::startVE. " + e.Message);
                    }
                }

                //change password
                try
                {
                    byte[] bytePassword = System.Text.ASCIIEncoding.ASCII.GetBytes(password);
                    string base64Password = Convert.ToBase64String(bytePassword);
                    object[] args = { ve_eid, "", System.Text.ASCIIEncoding.ASCII.GetBytes(base64Password) };
                    envmType.GetMethod("setUserPassword").Invoke(envmConnector, args).ToString();
                    Console.WriteLine("Thread {0}: Administrator password is set", threadIndex, ve_eid);
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to execute method Envm::setUserPassword. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method Envm::setUserPassword. " + e.Message);
                }

                // Logout
                try
                {
                    object[] args = {threadSession, url};
                    threadSession = connectType.GetMethod("Logout").Invoke(connectConnector, args).ToString();
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to logout. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to logout. " + e.Message);
                }

                Console.WriteLine("Thread {0}: successfully finished", threadIndex);
            }
            catch (Exception e)
            {
                errorMessage = e.Message;
                Console.WriteLine("Thread {0}: [ERROR] {1}", threadIndex, e);
                error = true;
            }

            doneEvent.Set();
        }
    }

    class DestroyVE : CreateMethodContainer
    {
        public bool error = false;
        public string ve_eid;
        public string errorMessage;

        private object url;
        private List<object> argsConnection;
        private ManualResetEvent doneEvent;

        public DestroyVE(object url_, List<object> argsConnection_, string ve_eid_, ManualResetEvent doneEvent_)
        {
            url = url_;
            argsConnection = argsConnection_;
            ve_eid = ve_eid_;
            doneEvent = doneEvent_;
            errorMessage = "No errors occurred";
        }

        public void destroyVEProc(Object threadContext)
        {
            int threadIndex = (int)threadContext;

            try
            {
                Console.WriteLine("Thread {0}: started...", threadIndex);

                string threadSession;
                Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");

                Type connectType = assembly.GetType("SOAPClasses.Connect");
                object connectConnector = createMethodContainer(connectType);

                // create per-thread session
                try
                {
                    threadSession = connectType.GetMethod("Login").Invoke(connectConnector, argsConnection.ToArray()).ToString();
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to create session. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to create session. " + e.Message);
                }

                object[] veidArray = {ve_eid};

                // stop VE            
                try
                {
                    Type type = assembly.GetType("SOAPClasses.Envm");
                    object connector = createMethodContainer(type, new object[] { url, threadSession });
                    ve_eid = type.GetMethod("stopVE").Invoke(connector, veidArray).ToString();

                    Console.WriteLine("Thread {0}: Successfully stopped CT {1}", threadIndex, ve_eid);
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to execute method Envm::stopVE. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method Envm::stopVE. " + e.Message);
                }

                // destroy VE            
                try
                {
                    Type type = assembly.GetType("SOAPClasses.Envm");
                    object connector = createMethodContainer(type, new object[] { url, threadSession });
                    ve_eid = type.GetMethod("destroyVE").Invoke(connector, veidArray).ToString();

                    Console.WriteLine("Thread {0}: Successfully destroyed CT {1}", threadIndex, ve_eid);
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to execute method Envm::destroyVE. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method Envm::destroyVE. " + e.Message);
                }

                // Logout
                try
                {
                    Type type = assembly.GetType("SOAPClasses.Connect");
                    object connector = createMethodContainer(type);
                    object[] args = {threadSession, url};
                    threadSession = type.GetMethod("Logout").Invoke(connector, args).ToString();
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to logout. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to logout. " + e.Message);
                }
            }
            catch (Exception e)
            {
                errorMessage = e.Message;
                Console.WriteLine("Thread {0}: [ERROR] {1}", threadIndex, e);
                error = true;
            }

            doneEvent.Set();
        }
    }

    class Program : CreateMethodContainer
    {
        static void parseParameters(string[] args, int start_pos)
        {
            for (; start_pos < args.Length; start_pos++)
            {
                if (args[start_pos].Split('=').Length == 2)
                    parameters[args[start_pos].Split('=')[0]] = args[start_pos].Split('=')[1];
                else
                    throw new ArgumentException("Unexpected command line argument - '" + args[start_pos] + "'");
            }
        }

        static int Main(string[] args)
        {
            try
            {
                if (args.Length == 0)
                {
                    throw new ArgumentException();
                }
                else
                {
                    switch (args[0])
                    {

                        case "-s":
                            showClasses();
                            break;

                        case "-m":
                            if (args.Length == 2)
                            {
                                showMethods(args[1]);
                            }
                            else if (args.Length < 2)
                            {
                                throw new ArgumentException("Class name is not specified");
                            }
                            else
                            {
                                throw new ArgumentException("Too many arguments");
                            }
                            break;

                        case "-e":
                            if (args.Length >= 3)
                            {
                                parseParameters(args, 3);
                                getSession();
                                execMethod(args[1], args[2]);
                                closeSession();
                            }
                            else
                            {
                                throw new ArgumentException("Class name and method are not specified");
                            }

                            break;

                        case "-t":
                            getTutorial();
                            break;
                        case "-q":
                            parseParameters(args, 1);
                            startHeavyLoadTest();
                            break;
                        case "/?":
                        default:
                            throw new ArgumentException();
                    }
                }

                return 0;
            }
            catch (System.ArgumentException e)
            {
                if (e.Message.Length > 0)
                    Console.WriteLine(e.Message);
                Console.WriteLine("Usage: SOAPconsole.exe {-s | -m class name| -e class name method [param_name=param_value [param_name=param_value ...]] | -t | -q [param_name=param_value [param_name=param_value ...]\n" +
                                    " -s: show all classes\n" +
                                    " -m: show methods of the specified class\n" +
                                    " -e: login and execute specified method\n" +
                                    " -t: start tutorial\n" + 
                                    " -q: start heavy-load test\n");
            }
            catch (ApplicationException e)
            {
                Console.WriteLine("[ERROR] " + e.Message);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
            closeSession();

            return -1;
        }


        static string session;
        static Hashtable parameters = new Hashtable();

        static void startHeavyLoadTest()
        {
            Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");

            // prepare list of connection arguments
            List<object> argsConnection = new List<object>();
            Type typeConnect = assembly.GetType("SOAPClasses.Connect");

            foreach (System.Reflection.ParameterInfo param in typeConnect.GetMethod("Login").GetParameters())
            {
                object val = "";
                if (parameters.ContainsKey(param.Name))
                {
                    val = parameters[param.Name].ToString();
                }
                else if (param.Name == "timeout")
                {
                    val = -1;
                }
                else if (param.Name == "realm" || param.Name == "domain")
                {
                    // empty values are possible
                }
                else
                {
                    Console.Write("Enter " + param.Name + " : ");
                    val = Console.ReadLine();
                }

                if (param.Name == "realm" && (Convert.ToString(val) == "" || Convert.ToString(val) == "system"))
                    val = "00000000-0000-0000-0000-000000000000";

                argsConnection.Add(val);
            }

            // check that all createVe parameters are specified
            Type typeEnvm = assembly.GetType("SOAPClasses.Envm");

            foreach (System.Reflection.ParameterInfo param in typeEnvm.GetMethod("createVE").GetParameters())
            {
                string line;
                if (parameters.ContainsKey(param.Name))
                {
                    line = parameters[param.Name].ToString();
                }
                else
                {
                    Console.Write("Enter " + param.Name + " : ");
                    parameters[param.Name] = Console.ReadLine();
                }
            }

            // check if password is provided
            string password = "1q";
            if(parameters.ContainsKey("user_password"))
                password = Convert.ToString(parameters["user_password"]);
            
            int threadCount = 10;
            if(parameters.ContainsKey("threads"))
                threadCount = Convert.ToInt32(parameters["threads"]);
            int maxThreadCount = 5;
            if (parameters.ContainsKey("activethreads"))
                maxThreadCount = Convert.ToInt32(parameters["activethreads"]);

            ManualResetEvent[] createVE_doneEvents = new ManualResetEvent[threadCount];
            CreateVE[] createVEArray = new CreateVE[threadCount];

            Console.WriteLine("# Starting {0} CreateVE threads (maximum {1} threads active at once)", threadCount, maxThreadCount);

            for(int i=0; i < threadCount; i++)
            {
                // allow to continue only if active threads count is less the maxThreadCount
                while (true)
                {
                    int runningThreads = 0;
                    ManualResetEvent[] createVE_doneEvents_ = new ManualResetEvent[i];
                    for (int j = 0; j < i; j++)
                    {
                        if (!createVE_doneEvents[j].WaitOne(0, false))
                            runningThreads++;
                        createVE_doneEvents_[j] = createVE_doneEvents[j];
                    }

                    if (runningThreads < maxThreadCount)
                        break;

                    WaitHandle.WaitAny(createVE_doneEvents_, 1000, false);
                }

                // prepare list of createVe parameters
                List<object> argsCreateVE = new List<object>();
                foreach (System.Reflection.ParameterInfo param in typeEnvm.GetMethod("createVE").GetParameters())
                {
                    if (param.Name == "name")
                    {
                        argsCreateVE.Add(parameters[param.Name].ToString() + "-" + Convert.ToString(i));
                    }
                    else if (param.Name == "hostname")
                    {
                        argsCreateVE.Add(parameters[param.Name].ToString() + "-" + Convert.ToString(i));
                    }
                    else if (param.Name == "ip")
                    {
                        string ip = parameters[param.Name].ToString();
                        string[] ip_ = ip.Split('.');
                        ip_[3] = Convert.ToString(Convert.ToInt16(ip_[3]) + i);
                        argsCreateVE.Add(String.Join(".", ip_));
                    }
                    else
                    {
                        argsCreateVE.Add(Convert.ChangeType(parameters[param.Name], param.ParameterType));
                    }
                }

                createVE_doneEvents[i] = new ManualResetEvent(false);
                CreateVE create = new CreateVE(parameters["url"], argsConnection, argsCreateVE, password, createVE_doneEvents[i]);
                createVEArray[i] = create;

                ThreadPool.QueueUserWorkItem(create.createVEProc, i);
            }

            WaitHandle.WaitAll(createVE_doneEvents);

            int errorNumber = 0;
            List<string> createdVEs = new List<string>();
            Console.WriteLine("# All CreateVE threads finished its work:");
            for (int i = 0; i < threadCount; i++)
            {
                if (createVEArray[i].ve_eid.Length > 0)
                    createdVEs.Add(createVEArray[i].ve_eid);

                if (createVEArray[i].error)
                    errorNumber++;

                Console.WriteLine("# Thread {0}", i);
                Console.WriteLine("#     " + createVEArray[i].ve_eid);
                Console.WriteLine("      " + createVEArray[i].errorMessage);
                Console.WriteLine();
            }

            Thread.Sleep(10000);

            // now destroy created CTs
            Console.WriteLine("# Starting {0} DestroyVE threads (maximum {1} threads active at once)", createdVEs.Count, maxThreadCount);

            ManualResetEvent[] destroyVE_doneEvents = new ManualResetEvent[createdVEs.Count];
            DestroyVE[] destroyVEArray = new DestroyVE[createdVEs.Count];


            for (int i = 0; i < createdVEs.Count; i++)
            {
                destroyVE_doneEvents[i] = new ManualResetEvent(false);
                DestroyVE destroy = new DestroyVE(parameters["url"], argsConnection, createdVEs[i], destroyVE_doneEvents[i]);
                destroyVEArray[i] = destroy;

                ThreadPool.QueueUserWorkItem(destroy.destroyVEProc, i);
            }

            WaitHandle.WaitAll(destroyVE_doneEvents);

            Console.WriteLine("# All DestroyVE threads finished its work.");

            for (int i = 0; i < destroyVEArray.Length; i++)
            {
                if (destroyVEArray[i].error)
                    errorNumber++;
            }

            // ok, print final statistics
            Console.WriteLine("# {0} errors totally during heavy load test.", errorNumber);
        }
        
        static void execMethod(string typeName, string methodName)
        {
            Console.WriteLine(execMethod_(typeName, methodName));
        }

        static string execMethod_(string typeName, string methodName)
        {
            if (session != null && session != "")
            {
                Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
                Type type = assembly.GetType("SOAPClasses." + typeName);

                if (type == null)
                    throw new ApplicationException("Failed to load class " + typeName);

                if (type.GetMethod(methodName) == null)
                    throw new ApplicationException("Method " + typeName + "::" + methodName + " does not exist");

                object connector = createMethodContainer(type, new object[] { parameters["url"], session });
                List<object> arguments = new List<object>();
                foreach (System.Reflection.ParameterInfo param in type.GetMethod(methodName).GetParameters())
                {
                    if (parameters.ContainsKey(param.Name))
                    {
                        arguments.Add(Convert.ChangeType(parameters[param.Name], param.ParameterType));
                    }
                    else
                    {
                        Console.Write("Enter " + param.Name + " : ");
                        string line = Console.ReadLine();

                        try
                        {
                            arguments.Add(Convert.ChangeType(line, param.ParameterType));
                        }
                        catch (Exception e)
                        {
                            throw new ApplicationException("Failed to parse value '" + line + "' provided for parameter '" + param.Name + "'." + e.Message);
                        }
                    }
                }

                try
                {
                   return type.GetMethod(methodName).Invoke(connector, arguments.ToArray()).ToString();
                }
                catch (ApplicationException e)
                {
                    if(e.InnerException != null)
                        throw new ApplicationException("Failed to execute method " + typeName + "::" + methodName +". " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to execute method " + typeName + "::" + methodName + ". " + e.Message);
                }
            }
            else
                throw new ApplicationException("Failed to execute method " + typeName + "::" + methodName + " - no session object yet.");

        }

        static void getTutorial()
        {
            try
            {
                Console.WriteLine("SOAP Tutorial");
                Console.WriteLine("\nFirst, make new session");
                Connect connection = new Connect();
                Console.Write("Enter VZ node url(http://some_ip:8080 or https://some_ip:4646):");
                string url = Console.ReadLine();
                Console.Write("Enter user:");
                string user = Console.ReadLine();
                Console.Write("Enter password:");
                string password = Console.ReadLine();
                string session = "";

                session = connection.Login(url, user, "", "00000000-0000-0000-0000-000000000000", password);
                Console.WriteLine("SessionID:" + session);

                Console.WriteLine("Let's make new container!\n");
                SOAPClasses.Binder binder = new SOAPClasses.Binder(url, session);
                Envm environment = new Envm(binder);

                Console.Write("Enter name:");
                string veName = Console.ReadLine();

                Console.Write("Enter platform(linux/windows):");
                string platform = Console.ReadLine();

                Console.Write("Enter architecture(i386/ia64/x86_64):");
                string arch = Console.ReadLine();

                Console.Write("Enter os template:");
                string template = Console.ReadLine();
                string veid = "";
                veid = environment.createVE(veName, template, environment.getSample("linux", "x86_64",""), platform, arch, veName, "10.0.0.2", "255.255.255.0", "venet0", false);
                Console.WriteLine("VEID: " + veid);

                Console.WriteLine("\nLet's get container's list");
                Console.WriteLine(environment.getVEList(3));

                Console.WriteLine("\nLet's get container's info");
                Console.WriteLine(environment.getConfig(veid));

                Console.WriteLine("\nLet's stop container");
                environment.stopVE(veid);

                Console.WriteLine("\nLet's destroy container");
                environment.destroyVE(veid);
            }
            catch (ApplicationException e)
            {
                throw new ApplicationException("Failed to process tutorial. " + e.InnerException.Message);
            }

        }

        #region for assembly
        static void getSession()
        {
            try
            {
                Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
                Type type = assembly.GetType("SOAPClasses.Connect");

                object connector = createMethodContainer(type);
                List<object> arguments = new List<object>();
                foreach (System.Reflection.ParameterInfo param in type.GetMethod("Login").GetParameters())
                {
                    string line;
                    if (parameters.ContainsKey(param.Name))
                    {
                        line = parameters[param.Name].ToString();
                    }
                    else
                    {
                        Console.Write("Enter " + param.Name + " : ");
                        line = Console.ReadLine();
                    }

                    if (param.Name == "realm" && (line == "" || line == "system"))
                        arguments.Add("00000000-0000-0000-0000-000000000000");
                    else
                        arguments.Add(line);
                }
                session = type.GetMethod("Login").Invoke(connector, arguments.ToArray()).ToString();
            }
            catch (ApplicationException e)
            {
                throw new ApplicationException("Failed to create session. " + e.InnerException.Message);
            }
        }

        static void closeSession()
        {
            if (session != null)
            {
                try
                {
                    Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
                    Type type = assembly.GetType("SOAPClasses.Connect");

                    object connector = createMethodContainer(type);
                    object[] args = { session, parameters["url"] };
                    type.GetMethod("Logout").Invoke(connector, args);
                    session = null;
                }
                catch (ApplicationException e)
                {
                    if (e.InnerException != null)
                        throw new ApplicationException("Failed to close session. " + e.InnerException.Message);
                    else
                        throw new ApplicationException("Failed to close session. " + e.Message);
                }
            }
        }

        static void showMethods(string typeName)
        {
            Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
            Type type = assembly.GetType("SOAPClasses." + typeName);
            Console.WriteLine("Creating method container...");
            object connector = createMethodContainer(type);
            Console.WriteLine(typeName + " has methods:");
            foreach (MethodInfo method in type.GetMethods())
                Console.WriteLine(" [" + method.Name + "]");

        }

        static void showClasses()
        {
            Assembly assembly = Assembly.LoadFrom("SOAPClasses.dll");
            Console.WriteLine("Samples include:");
            foreach (Type typeInfo in assembly.GetTypes())
            {
                if (!typeInfo.FullName.Contains(".VZA") && !typeInfo.FullName.Contains("Trust") && !typeInfo.FullName.Contains("Settings"))
                    Console.WriteLine(" [" + typeInfo.Name + "]");
            }
        }
        
        #endregion
    }
}
